; DIGIREC.ASM				(R. CUTLER,  04/30/91)
; Copyright (C) 1991 by Cutler Digital Design

   ;************************************************************
   ;***                      INCLUDES
   ;************************************************************

	include lmacros.h
	INCLUDE	DIGIREC.EQU


   ;************************************************************
   ;***                      MACROS                          ***
   ;************************************************************

   ;***  usage OUTPT  OUTPUT-PORT , VALUE

OUTPT	MACRO  ARG1,ARG2  ;***  Allows output of reg., mem. or constants
	IF	ARG1 GT 255       ;***  Used by lots of routines
	IFDIF	<"DX">,<ARG1>
	MOV     DX,ARG1
	ENDIF
	IFDIF	<"AL">,<ARG2>
	MOV     AL,ARG2
	ENDIF
	OUT     DX,AL
	ELSE
	IFDIF	<"AL">,<ARG2>
	MOV	AL,ARG2
	ENDIF
	OUT	ARG1,AL
	ENDIF
        ENDM


   ;***  usage OUTPTL  OUTPUT-PORT(>255) , VALUE

OUTPTL	MACRO  ARG1,ARG2  ;***  Allows output of reg., mem. or constants
	MOV     DX,ARG1
	MOV     AL,ARG2
	OUT     DX,AL
        ENDM


   ;***  usage INPUT  INPUT-PORT

INPUT	MACRO  ARG1       ;***  Allows input to reg., mem. or constants
	IF	ARG1 GT 255       ;***  Used by lots of routines
	IFDIF	<"DX">,<ARG1>
        MOV     DX,ARG1
	ENDIF
        IN      AL,DX
	ELSE
	IN	AL,ARG1
	ENDIF
        ENDM


INTON	MACRO         ;***  Enable interrupts
	STI              ;***  (Used by InitIO & GITCH3)
        ENDM


INTOFF	MACRO      ;***  Disable interrupts 
	CLI              ;***  (InitIO, DOEDGE & GITCH1)
        ENDM



;*********************************
;
; DATA STRUCTURE USED INT THE SERIN ROUTINES
;   CALLED ONCE FOR EACH SERIAL PORT
;   THESE VARIABLES ARE DECLARED IN SERIN0.ASM AND SERIN8.ASM
;   BY CALLING THE MACRO DEFINED IN SERIN.MAC
;
;*********************************
UDAT	MACRO	UN
DATASEG	SEGMENT	WORD PUBLIC 'data'
	EXTRN	STATE&UN&_:WORD		;HOLDS THE ADDRESS OF THE PORTION OF
					; CODE TO EXECUTE ONCE AN INPUT
					; CHARACTER IS FOUND FOR THIS CHANNEL
					; DURING THE MILLISECOND INTERRUPT
	EXTRN	STCHAR&UN&_:BYTE	;SYNC CHARACTER, USED TO RECOGNIZE WHEN
					; A NEW DATA FRAME (8 CHARS, 7 WITH DATA)
					; IS ABOUT TO ARRIVE
	EXTRN	STSTATE&UN&_:WORD	;MEMORY OF WHAT THE STARTING STATE IS
					; CURRENTLY.  THIS IS NECESSARY, SINCE
					; STATE INFORMATION INCLUDES THE CHOICE
					; OF WHICH BUFFER DATA ARE SUPPOSED TO
					; GO TO.  WHEN AN RECEIVER ERROR IS DETECTED
					; THE CODE WILL RESET TO THE BEGINNING
					; OF THE BUFFER
	EXTRN	EMPBUF&UN&_:WORD	;MISSING DATA BUFFER
	EXTRN	SHB0&UN&_:WORD		;DATA BUFFER 0
	EXTRN	SHB1&UN&_:WORD		;DATA BUFFER 1
	EXTRN	SHB2&UN&_:WORD		;DATA BUFFER 2
	EXTRN	IBFCNT&UN&_:WORD	;INPUT BUFFER COUNT, EITHER 0,2,OR 4
					; IS USED TO DECIDE WHICH INPUT BUFFER
					; TO USE, WHEN THE SAMPLE INTERVAL IS
					; OVER, AND DATA ARE TRANSFERRED FROM
					; INPUT BUFFERS, TO THE LARGE ARRAY
					; OF HOLDING BUFFERS
	EXTRN	EMPPTR&UN&_:WORD	;POINTER TO MISSING DATA
	EXTRN	CURPTR&UN&_:WORD	;POINTER TO MOST RECENTLY TRANSFERRED
					; FILLED BUFFER
	EXTRN	PREVPTR&UN&_:WORD	;POINTER TO OLDEST RESIDENT INPUT
					; BUFFER
	EXTRN	BRATE&UN&_:BYTE		;BAUD RATE INDICATOR FOR UART
	EXTRN	PARITY&UN&_:BYTE	;PARITY INDICATOR FOR UART
	EXTRN	FRSTCH&UN&_:WORD	;FLAG TO INDICATE WHETHER THE FIRST
					; DATUM IN THE BUFFER IS ENABLED
					; NOT CURRENTLY USED
	EXTRN	SCNDCH&UN&_:WORD	;FLAG TO INDICATE WHETHER THE SECOND
					; DATUM IN THE BUFFER IS ENABLED
					; NOT CURRENTLY USED
	EXTRN	THRDCH&UN&_:WORD	;FLAG TO INDICATE WHETHER THE THIRD
					; DATUM IN THE BUFFER IS ENABLED
					; NOT CURRENTLY USED
	EXTRN	FRTHCH&UN&_:WORD	;FLAG TO INDICATE WHETHER THE FOURTH
					; DATUM IN THE BUFFER IS ENABLED
					; NOT CURRENTLY USED
	EXTRN	FRSTOFF&UN&_:WORD	;OFFSET OF THE FIRST DATUM IN
					; THE HOLDING BUFFERS.  SINCE THERE
					; ARE AT MOST 64 DATA POINTS, A VALUE
					; OF 64*2 CAUSES THIS DATA POINT TO
					; BE STORED BEYOND THE HOLDING BUFFER
					; AND EFFECTIVELY DISABLES IT.
	EXTRN	SCNDOFF&UN&_:WORD	;OFFSET OF THE SECOND DATUM IN
					; THE HOLDING BUFFERS.  SINCE THERE
					; ARE AT MOST 64 DATA POINTS, A VALUE
					; OF 64*2 CAUSES THIS DATA POINT TO
					; BE STORED BEYOND THE HOLDING BUFFER
					; AND EFFECTIVELY DISABLES IT.
	EXTRN	THRDOFF&UN&_:WORD	;OFFSET OF THE THIRD DATUM IN
					; THE HOLDING BUFFERS.  SINCE THERE
					; ARE AT MOST 64 DATA POINTS, A VALUE
					; OF 64*2 CAUSES THIS DATA POINT TO
					; BE STORED BEYOND THE HOLDING BUFFER
					; AND EFFECTIVELY DISABLES IT.
	EXTRN	FRTHOFF&UN&_:WORD	;OFFSET OF THE FOURTH DATUM IN
					; THE HOLDING BUFFERS.  SINCE THERE
					; ARE AT MOST 64 DATA POINTS, A VALUE
					; OF 64*2 CAUSES THIS DATA POINT TO
					; BE STORED BEYOND THE HOLDING BUFFER
					; AND EFFECTIVELY DISABLES IT.
DATASEG	ENDS
	ENDM

;*********************************
;
; INIT DATA STRUCTURE FOR UART &UNUM
;
;*********************************
INITU	MACRO	UNUM
	EXTRN	SI00&UNUM:NEAR
	MOV	STATE&UNUM&_,OFFSET SI00&UNUM	;ADDRESS OF CODE TO LOOK FOR
						; SYNC CHARACTER DURING
						; MSECOND INTERRUPT PROCESSING

	MOV	STSTATE&UNUM&_,OFFSET SI00&UNUM	;SET RESET ADDRESS TO THE SAME
	MOV	STCHAR&UNUM&_,STARTBYTE		;STANDARD START BYTE

	MOV	BRATE&UNUM&_,R9600		;SET STANDARD BAUD RATE

	MOV	PARITY&UNUM&_,ODDP		;SET TO ODD PARITY
	MOV	FRSTOFF&UNUM&_,(0&UNUM&H * 8)	;INIT OFFSET FIRST CHANNEL
	MOV	SCNDOFF&UNUM&_,(0&UNUM&H * 8)+2	;INIT OFFSET SECOND CHANNEL
	MOV	THRDOFF&UNUM&_,(0&UNUM&H * 8)+4	;INIT OFFSET THIRD CHANNEL
	MOV	FRTHOFF&UNUM&_,(0&UNUM&H * 8)+6	;INIT OFFSET FOURTH CHANNEL
	MOV	EMPBUF&UNUM&_,08000H		;INIT MISSING DATA BUFFER
	MOV	EMPBUF&UNUM&_+2,08000H
	MOV	EMPBUF&UNUM&_+4,08000H
	MOV	EMPBUF&UNUM&_+6,08000H
	MOV	IBFCNT&UNUM&_,0			;INIT INPUT BUFFER COUNT
	MOV	EMPPTR&UNUM&_,OFFSET EMPBUF&UNUM&_  ; INIT THE POINTER TO
						    ;  MISSING DATA BUFFER

	ENDM

;***********************************
;
; ENABLE UART &UNUM1 (SEE DATA SHEETS FOR 2681 DUAL UART FOR DETAILS)
;
;***********************************
ENABLEU	MACRO	UNUM1
	PUBLIC	ENABLEU&UNUM1&_
ENABLEU&UNUM1&_:	PUSH	AX
	MOV	AL,00111010B
	OUT	UART&UNUM1+UCOMM,AL	;RESET TRANSMIT & DISABLE TX AND RX
	MOV	AL,00101010B
	OUT	UART&UNUM1+UCOMM,AL	;RESET RECVR & DISABLE TX AND RX
	MOV	AL,00011010B
	OUT	UART&UNUM1+UCOMM,AL	;RESET MR PTR &DISABLE TX AND RX
	MOV	AL,PARITY&UNUM1&_
	OUT	UART&UNUM1+UMODE,AL	;SET PARITY
	MOV	AL,00000111B
	OUT	UART&UNUM1+UMODE,AL	;SET MODE AND STOP BITS
	MOV	AL,BRATE&UNUM1&_
	OUT	UART&UNUM1+CSR,AL	;SET BAUD RATE (ACR MUST BE SET ELSEWHERE)
	MOV	AL,1
	OUT	UART&UNUM1+UCOMM,AL	;TURN UART ON
	POP	AX
	RET
	ENDM

;*************************************
;
; RESET DUART &DNUM (SEE DATA SHEETS FOR 2681 DUAL UART FOR DETAILS)
;
;*************************************
INITD	MACRO	DNUM
	MOV	AL,0
	OUT	DUART&DNUM+OPCR,AL	;PURE OUTPUT CONTROL
	OUT	DUART&DNUM+ACR,AL	;DEFAULT
	OUT	DUART&DNUM+IMR,AL	;DEFAULT
	MOV	AL,0FFH
	OUT	DUART&DNUM+RESET,AL	;CLEAR
	ENDM

;**************************************
;
;	DISABLE RCVER OF UART&UNUM3 (SEE DATA SHEETS FOR 2681 DUAL UART)
;
;**************************************
DISABLEU MACRO	UNUM3
	PUBLIC	DISABLEU&UNUM3&_
DISABLEU&UNUM3&_:	MOV	AL,2
	OUT	UART&UNUM3+UCOMM,AL
	RET
	ENDM

;**************************************
;
;	GET DATA FROM INPUT BUFFERS INTO
;		CAPTURE  BUFFER
;
;**************************************
GETDATA	MACRO	U
	LOCAL	DOWN
	MOV	SI,IBFCNT&U&_	; GET INPUT BUFFER COUNT, WHICH IS IN FACT AN
				;  OFFSET INTO A POINTER TABLE WHOSE BASE IS
				;  EMPPTR&U&_
	MOV	SI,EMPPTR&U&_[SI];GET THE ACTUAL DATA
				 ; POINTER IN SI
; THE FOLLOWING LINE IS INCLUDED IF NO MISSING DATA IS WANTED
	MOV	EMPPTR&U&_,SI	;SAVE THIS POINTER FOR FUTURE MISSING DATA

; GET THE FIRST DATUM, ASSUME DX IS ALREADY SET TO THE DESTINATION
	MOV	DI,DX
	ADD	DI,FRSTOFF&U&_
	MOVS	CAPDATA_,EMPBUF&U&_

; GET THE SECOND DATUM, ASSUME DX IS ALREADY SET TO THE DESTINATION
	MOV	DI,DX
	ADD	DI,SCNDOFF&U&_
	MOVS	CAPDATA_,EMPBUF&U&_

; GET THE THIRD DATUM, ASSUME DX IS ALREADY SET TO THE DESTINATION
	MOV	DI,DX
	ADD	DI,THRDOFF&U&_
	MOVS	CAPDATA_,EMPBUF&U&_

; GET THE FOURTH DATUM, ASSUME DX IS ALREADY SET TO THE DESTINATION
	MOV	DI,DX
	ADD	DI,FRTHOFF&U&_
	MOVS	CAPDATA_,EMPBUF&U&_

;ADJUST THE INPUT BUFFER COUNT
; INTERRUPT SHOULD ALREADY BE DISABLED, SO THERE WILL BE NO INTERFERENCE
; FROM THE MILLISECOND INTERRUPT ROUTINE
	SUB	IBFCNT&U&_,2
	JNS	DOWN
	MOV	IBFCNT&U&_,0
DOWN:



	ENDM
	

   ;************************************************************
   ;***                      EQUATES, AND DECLARATIONS
   ;************************************************************

;COMMENTS FROM MANX C SETUP:
;
;	If you want your stack at a fixed place, you may remove the
;	"bss cstack..." statement below and change the "mov sp,cstack..."
;	to load SP with the value you desire.  Note that the setup of
;	SS may need to be changed also.  If the program is small data
;	model, then SS must be equal to DS or pointers to automatic
;	variables won't work.
;
;	Otherwise, stacksize should be set according to your program's
;	requirements.
stacksize equ	2048
;
;	DATASEG is the data segment address (as a paragraph #)
;	this is picked from the -D option of the linker
;
;Dseg	equ	040H	;physical addr 0x400, just above the int vectors

; IT IS A LITTLE TRICKY ASSIGNING LOWER RAM, IT SEEMS THAT THE FIRST BYTE
;   IN dataseg IS RESERVED FOR _DORG_.  INSERTING ORG STATMENTS SEEMS TO
;   ENFORCE A TWO BYTE OFFSET.

; INTERRUPT VECTORS:
DATASEG	segment	word public 'data'

	DW	?
	PUBLIC	VEC01
VEC01	DW	?
	DW	?
VEC02	DW	?
	DW	?
VEC03	DW	?
	DW	?
VEC04	DW	?
	DW	?
VEC05	DW	?
	DW	?
VEC06	DW	?
	DW	?
VEC07	DW	?
	DW	?
VEC08	DW	?
	DW	?
VEC09	DW	?
	DW	?
VEC0A	DW	?
	DW	?
VEC0B	DW	?
	DW	?
VEC0C	DW	?
	DW	?
VEC0D	DW	?
	DW	?
VEC0E	DW	?
	DW	?
VEC0F	DW	?
	DW	?
VEC10	DW	?
	DW	?
VEC11	DW	?
	DW	?
VEC12	DW	?
	DW	?
VEC13	DW	?
	DW	?
VEC14	DW	?
	DW	?
VEC15	DW	?
	DW	?
VEC16	DW	?
	DW	?
VEC17	DW	?
	DW	?
VEC18	DW	?
	DW	?
VEC19	DW	?
	DW	?
VEC1A	DW	?
	DW	?
VEC1B	DW	?
	DW	?
VEC1C	DW	?
	DW	?
VEC1D	DW	?
	DW	?
VEC1E	DW	?
	DW	?
VEC1F	DW	?
	DW	?

	PUBLIC	VECT0
VECT0	DW	OFFSET MSINT
	DW	CODESEG
	PUBLIC	VECSER
VECSER	DW	?
	DW	?
	PUBLIC	VECT2
VECT2	DW	?
	DW	?
VECFR0	DW	?
	DW	?
VECFR1	DW	?
	DW	?
VECFR2	DW	?
	DW	?
	PUBLIC	VECXCLK
VECXCLK	DW	OFFSET SAMPINT
	DW	CODESEG
	PUBLIC	VECCLK
VECCLK	DW	OFFSET SAMPINT
	DW	CODESEG

;MANX C DECLARATIONS
	public	$MEMRY
$MEMRY	dw	-1
	dw	-1
	public	errno_
errno_	dw	0
	public	_dsval_,_csval_
_dsval_	dw	0
_csval_	dw	0
	public	_mbot_, sbot
_mbot_	dw	0
	dw	0
sbot	dw	0
	dw	0
	extrn	_Dorg_:byte,_Dend_:byte
	extrn	_Uorg_:byte,_Uend_:byte
	public	cstack
	bss	cstack:byte,stacksize
DATASEG	ends
	extrn	_Corg_:byte,_Cend_:byte

ifdef	FARPROC
	extrn main_:far, $fltinit:far
else
	extrn main_:near, $fltinit:near
endif

;MANX C STARTUP CODE
;____________________________________________________________________
;____________________________________________________________________
;
;/////////////////   STARTUP ROUTINE CODE SPACE   \\\\\\\\\\\\\\\\\\\
;____________________________________________________________________
;____________________________________________________________________

	public	$begin
$begin	proc	far
	cli
	cld
; initialize stack
	mov	bx,DATASEG	;place where data is to go in ram
	mov	ds,bx
	mov	es,bx
;
;	Note: For hardware reasons the instruction which loads SS should
;		be immediatly followed by the load of SP.
;
	mov	ss,bx
	mov	sp,offset cstack+stacksize

;************************************************************
;
; V40 INIT CODE
;
;************************************************************
	ASSUME CS:CODESEG,DS:DATASEG,ES:DATASEG,SS:DATASEG




;P1BASE
; SET UP ON CHIP PERIPHERAL CONNECTION REGISTER (OPCN)
;
; BITS: 7 6 5 4 |  3 2 | 1 0 |
;       - - - - | IRSW | PF  |
;     IRSW = 00	INTP1 PIN IS INT1, INTP2 PIN IS INT2
;	   = 01 SCU INT IS INT1, INTP2 PIN IS INT2
;	   = 10 INTP1 PIN IS INT1, TOUT2 IS INT2
;	   = 11 SCU INT IS INT1, TOUT2 IS INT2
;
;     PF   = 00 DMARQ3, DMAAK3-, INTAK-
;	   = 01 DMARQ3, DMAAK3-, TOUT1
;	   = 10 RxD, TxD, INTAK-
;	   = 11 RxD, TxD, SRDY-
;
	OUTPTL	OPCN,BIT3+BIT2+BIT0	

; SET UP ON CHIP PERIPHERAL SELECTION REGISTER (OPSEL)
; BITS: 7 6 5 4 |  3 |  2 |  1 |  0 |
;       - - - - | SS | TS | IS | DS |
;
;	SS = 0 DISABLE SERIAL CONTROL UNIT
;	SS = 1 ENABLE SERIAL CONTROL UNIT
;	TS = 0 DISABLE TIMER CONTROL UNIT
;	TS = 1 ENABLE TIMER CONTROL UNIT
;	IS = 0 DISABLE INTERRUPT CONTROL UNIT
;	IS = 1 ENABLE INTERRUPT CONTROL UNIT
;	DS = 0 DISABLE DMA CONTROL UNIT
;	DS = 1 ENABLE DMA CONTROL UNIT
;	OUTPTL	OPSEL,BIT2+BIT1+BIT0	;ENABLE DMA,INTERRUPT AND TIMER UNITS
	OUTPTL	OPSEL,BIT2+BIT1		;FOR NOW, ONLY TIMER UNIT AND ICU


;	SET OPHA TO 00H
	OUTPTL	OPHA,IPBASE

;	SET UP INTERNAL PERIPHERAL ADDRESSES
	OUTPTL	DULA,DMABASE
	OUTPTL	IULA,IUBASE
	OUTPTL	TULA,TUBASE
	OUTPTL	SULA,SUBASE



; TIMER CLOCK SELECTION REGISTER SETUP
; BITS: 7 6 5 |  4  |  3  |  2  |  1  |  0  |
;       - - - | CS2 | CS1 | CS0 |     PS    |
;
;	PS = 00 PRESCALE BY 2
;	PS = 01 PRESCALE BY 4
;	PS = 10 PRESCALE BY 8
;	PS = 11 PRESCALE BY 16
;
;	CSN = 0	CLOCK TO TIMER N = INTERNAL CLOCK
;	CSN = 1 CLOCK TO TIMER N = TCLK PIN
	OUTPTL	TCKS,00010000B		;PRESCALE=2, ONLY T2 DRIVEN EXTERNALLY

; SET UP REFRESH CONTROL REGISTER
; BITS: |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
;	| RE  |  -  |  -  |             RTM             |
;
;	RE = 0 DISABLES REFRESH
	OUTPTL	RFC,0H

; SET UP MEMORY BOUNDARY REGISTER
; BITS: |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
;       |  -  |       LMB       |  -  |       UMB       |
;
;	LMB AND UMB BLOCK SIZE
;	000	32KB
;	001	64KB
;	010	96KB
;	011	128KB
;	100	192KB
;	101	256KB
;	110	384KB
;	111	512KB
	OUTPTL	WMB,0H		;SET EACH SIZE AT 32KB

; SET UP WAIT CYCLES FOR DIFFERENT PARTS OF MEM AND I/0 (WCY1)
; BITS: |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
;       |    IOW    |    UMW    |    MMW    |    LMW    |
;
;	XXW =	00	0 WAIT STATES
;		01	1 WAIT STATE
;		10	2 WAIT STATES
;		11	3 WAIT STATES
	OUTPTL	WCY1,11000000B	; SET RAM/ROM TO 0 WAIT STATES
				;     I/O TO 3 WAIT STATES
				; MAYBE CHANGE LATER
; SET UP WAIT CYCLES FOR DMA AND REFRESH (WCY2)
; BITS: |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
;       |  -  |  -  |  -  |  -  |   DMAW    |    RFW    |
;
;	XXW =	00	0 WAIT STATES
;		01	1 WAIT STATE
;		10	2 WAIT STATES
;		11	3 WAIT STATES
	OUTPTL	WCY2,00000000B	;SET EVERYTHING TO 0 WAIT STATES
				; TUNE THIS LATER IF USED

; SET UP INTERRUPT REGISTER
; INITIALIZE INTERRUPT CONTROLLER
	OUTPT	INTREG0,00010011B	; INITIALIZE,EDGE TRIG,SINGLE MODE
	OUTPT	INTREG1,00100000B	;VECTOR TABLE STARTS AT ADD 80H
	OUTPT	INTREG1,00000011B	;SELF FI MODE

	MOV	AL,0FFH		;SET MASK VECTOR
	CALL	MASKINT

; THE FOLLOWING CLOCK MUST BE STARTED BEFORE INITIALIZING UARTS, FOR
;  PROPER INIT:

; SET UP TIMER 2, IT USES TCLK (7.37mhz) AND DIVIDES IT BY TWO
;   SEE V40 TECHNICAL LITERATURE
	MOV	AL,10110100B	;REG2,LOWER BYTE THEN HIGHER,MODE2,BINARY
	OUT	CLKCON,AL
	MOV	AL,2
	OUT	CLK2,AL
	MOV	AL,0
	OUT	CLK2,AL

; INITIALIZE UART DATA STRUCTURES
	INITU	0
	INITU	1
	INITU	2
	INITU	3
	INITU	4
	INITU	5
	INITU	6
	INITU	7
	INITU	8
	INITU	9
	INITU	A
	INITU	B
	INITU	C
	INITU	D
	INITU	E
	INITU	F

; INITIALIZE DUART DATA STRUCTURES
	INITD	0
	INITD	1
	INITD	2
	INITD	3
	INITD	4
	INITD	5
	INITD	6
	INITD	7

; ENABLE UARTS
	CALL	ENABLEU0_
	CALL	ENABLEU1_
	CALL	ENABLEU2_
	CALL	ENABLEU3_
	CALL	ENABLEU4_
	CALL	ENABLEU5_
	CALL	ENABLEU6_
	CALL	ENABLEU7_
	CALL	ENABLEU8_
	CALL	ENABLEU9_
	CALL	ENABLEUA_
	CALL	ENABLEUB_
	CALL	ENABLEUC_
	CALL	ENABLEUD_
	CALL	ENABLEUE_
	CALL	ENABLEUF_

;  INIT DMA TO PC OPERATIONS
;1234
	MOV	DMAPTR_,OFFSET BUFFER_
	MOV	DMACNT_,0FFFFH
	MOV	CAPFLAG_,0H
	MOV	AL,DMAON+DMA0+DMA1
	OUT	DMAPORT+RESET,AL
	OUT	DMAREG+1,AL
;  INIT SAMPLE BUFFERING STRUCTURE, SET TO SAMPLE INTO ONLY 2 BUFFER
	MOV	AX,OFFSET BUFFER_
	MOV	SBIPTR_,AX
	MOV	SBOPTR_,AX
	MOV	SBBPTR_,AX
	ADD	AX,256
	MOV	SBEPTR_,AX
	MOV	SBINC_,128
	MOV	SBCNT_,0
	MOV	DMAMXCNT_,64
	MOV	SBMXCNT_,2


; SET UP TIMER 1, IT USES INTERNAL CLOCK (3.68 mhz) AND DIVIDES IT BY 24
;					TO GET THE APPROP FREQ FOR 9600
	MOV	AL,01110100B	;REG2,LOWER BYTE THEN HIGHER,MODE2,BINARY
	OUT	CLKCON,AL
	MOV	AL,24
	OUT	CLK1,AL
	MOV	AL,0
	OUT	CLK1,AL

; SET UP TIMER 0, IT USES INTRNL CLK (3.68 mhz) AND DIVIDES TO 1MS
	MOV	AL,00110100B	;REG2,LOWER BYTE THEN HIGHER,MODE2,BINARY
	OUT	CLKCON,AL
	MOV	AL,LOW 3686
	OUT	CLK0,AL
	MOV	AL,HIGH 3686
	OUT	CLK0,AL

;SET UP MILLISECOND INTERRUPT VECTOR
	MOV	AX,CODESEG
	MOV	VECT0+2,AX
	MOV	AX,OFFSET MSINT
	MOV	VECT0,AX

;SET UP MILLISECOND ADJUSTMENT REGISTERS
	MOV	BASECNT0_,3686
	MOV	BASECNT1_,3687
	MOV	REMDCNT,0
	MOV	CNTINC_,26214		;((65536.0*4.0)/10.0)
	MOV	PERIOD_,10
	MOV	MCOUNT_,10	;RESET MCOUNT

; CLEAR THE MS INTERRUPT FLAG
	MOV	MSINTFLG,0

;ENABLE T0 INTERRUPT
	MOV	AL,00000001B
	CALL	UNMASKINT
	MOV	AL,01000000B
	CALL	UNMASKINT
; END V40 INIT CODE
;

;**************************************************************
;
;	BACK TO MANX C STARTUP CODE
;
;**************************************************************

	cli
	cld
;
;	Compute where initialzed data starts (@ next para after code)
;
	mov	ax,offset _Cend_+15
	mov	cl,4
	shr	ax,cl
	add	ax,seg _Cend_
	mov	ds,ax		;place where data is in rom
	mov	bx,DATASEG	;place where data is to go in ram
	mov	es,bx
;
;	Note: For hardware reasons the instruction which loads SS should
;		be immediatly followed by the load of SP.
;
	mov	ss,bx
	mov	sp,offset cstack+stacksize
;
;		copy Init data from rom to ram
	mov	di,0
	mov	cx,offset _Dend_
	inc	cx
	shr	cx,1
	jcxz	nocopy
	mov	si,0
rep	movsw
nocopy:
;
;		clear uninitialized data

;**** don't zero uninitialized stuff
	jmp	noclear
;****
	mov	di,offset _Uorg_
	mov	cx,offset _Uend_
	sub	cx,di
	inc	cx
	shr	cx,1
	jcxz	noclear
	sub	ax,ax
rep	stosw
noclear:
;
	assume	ds:DATASEG,es:DATASEG

	mov	ds,bx			;set DS, now DS, SS, ES are equal
	mov	di,$MEMRY
	inc	di
	and	di,0fffeH	;adjust to word boundary
	mov	$MEMRY,di	;save memory allocation info for sbrk()
	mov	$MEMRY+2,ds
	mov	_mbot_,di
	mov	_mbot_,ds
	mov	sbot,0ffffH	;this is the heap limit for sbrk()
	mov	sbot+2,0fff0h
	mov	_dsval_,ds
	mov	_csval_,cs	;this is of dubious value in large code
	sti
	call	$fltinit	;setup floating point software/hardware
	jnc	flt_ok
	hlt			;program needs 8087 and one wasn't found
flt_ok:

	jmp	main_		;main shouldn't return in ROM based system
$begin	endp


	ASSUME CS:CODESEG,DS:DATASEG,ES:DATASEG,SS:DATASEG


;*********************************************************
;
;	DEFINE THE ENABLE/DISABLE UART ROUTINES
;
;*********************************************************
	ENABLEU	0
	ENABLEU	1
	ENABLEU	2
	ENABLEU	3
	ENABLEU	4
	ENABLEU	5
	ENABLEU	6
	ENABLEU	7
	ENABLEU	8
	ENABLEU	9
	ENABLEU	A
	ENABLEU	B
	ENABLEU	C
	ENABLEU	D
	ENABLEU	E
	ENABLEU	F

	DISABLEU	0
	DISABLEU	1
	DISABLEU	2
	DISABLEU	3
	DISABLEU	4
	DISABLEU	5
	DISABLEU	6
	DISABLEU	7
	DISABLEU	8
	DISABLEU	9
	DISABLEU	A
	DISABLEU	B
	DISABLEU	C
	DISABLEU	D
	DISABLEU	E
	DISABLEU	F

;*************************************************************
;
;		OTHER SERVICE ROUTINES
;
;*************************************************************
UNMASKINT: XOR	AL,0FFH		;COMPLEMENT AL
	AND	AL,INTMASK
	JMP	SHORT MASKINT1
MASKINT: OR	AL,INTMASK
MASKINT1: MOV	INTMASK,AL
	OUTPT	INTREG1,AL
	RET

;*************************************************************
;***                     procdef's                         ***
;***                                                       ***
;*************************************************************

; PERFORM INPUT
procdef	IOin	<<IOIport,word>>
	MOV	DX,IOIport
	IN	AL,DX
	MOV	AH,0
	pret
	pend	IOin

; PERFORM OUTPUT
procdef	IOout	<<IOOport,word>,<ODATA,word>>
	MOV	AX,ODATA
	MOV	DX,IOOport
	OUT	DX,AL
	pret
	pend	IOout

;GET ADDRESS OF START OF DMA (HOLDING) BUFFERS
procdef	GetPoint
	MOV	AX,OFFSET BUFFER_
	pret
	pend	GetPoint

;TEMPORARILY DISABLE INTERRUPTS
procdef INTOFF
	INTOFF
	pret
	pend	INTOFF

;ENABLE INTERRUPTS
procdef INTON
	INTON
	pret
	pend	INTON

;FORCE A HARD RESET
procdef	ColdReset
	JMP	$begin
	pret
	pend	ColdReset


;PROCESS A DMA TRANSFER TO PC, IF NECESSARY
procdef	DODMA
	PUSH	AX
	PUSH	BX
	CMP	DMACNT_,0FFFFH
	JE	DODMAX
DODMA1:	CMP	DMACNT_,0
	JNE	DODMA2		;IF DMACNT_ IS NOT ZERO, WE SEE IF DMA IS RDY

; ANY BUFFERS THERE?
	CMP	SBCNT_,0	;CHECK SBCNT_
	JE	DODMAX		;IF ZERO, EXIT
;BUFFERS THERE!

;TIME TO LOAD UP A NEW DMA BUFFER
    ;DECREASE BUFFER COUNT
	DEC	SBCNT_
    ;LOAD NEW DMA POINTER FROM OUTPUT BUFFER POINTER
	MOV	AX,SBOPTR_
	MOV	DMAPTR_,AX
    ;ADVANCE OUTPUT BUFFER POINTER AND WRAP IT IF NECESSARY
	MOV	AX,SBOPTR_	;GET POINTER IN AX
	ADD	AX,SBINC_	;ADJUST IT
	CMP	AX,SBEPTR_	;CMPARE WITH MAX+1
	JNE	DODMA4		;JUMP IF OKAY
	MOV	AX,SBBPTR_	;NOT OKAY, SO WRAP
DODMA4:	MOV	SBOPTR_,AX	; NOW STORE POINTER
    ;RESET THE DMA COUNT TO THE BUFFER SIZE
	MOV	AX,DMAMXCNT_
	MOV	DMACNT_,AX


DODMA2:
; IS DMA READY?
	IN	AL,DUART2+INREG
	AND	AL,01H		;LOOK AT DMA DONE BIT
	JNE	DODMA3		; IF NON ZERO, THEN READY, SO JMP TO RDY CODE
DODMAX:
	POP	BX
	POP	AX
	pret

DODMA3:
; GREAT, READY FOR DMA
	MOV	BX,DMAPTR_	;GET POINTER TO DATA
	MOV	AX,[BX]		;GET ACTUAL DATA
	OUT	DMAREG,AX	;DMA DONE, NOW UPDATE
	DEC	DMACNT_		;DECREASE COUNT
	ADD	DMAPTR_,2	;ADVANCE POINTER
	JMP	DODMA1		; AND LOOP BACK TO TRY AGAIN

	pend	DODMA


   ;*************************************************************
   ;***              INTERRUPT CALLS                          ***
   ;*************************************************************



;SAMPLE INTERVAL INTERRUP ROUTINE
SAMPINT:
	PUSH	AX
	PUSH	DX
	PUSH	SI
	PUSH	DI

;DUART6 SHOWS THE BUFFER COUNT ON ITS OUTPUT (FIRST 8 BITS)
	MOV	DX,DUART6+RESET
	MOV	AL,255
	OUT	DX,AL
	MOV	DX,DUART6+SET
	MOV	AX,SBCNT_
	OUT	DX,AL

;CHECK TO SEE IF THE CURRENT SAMPLED BUFFER IS TO GO INTO THE
; DMA HOLDING BUFFERS,  OR INTO THE CAPTURE BUFFER
; * IN NORMAL OPERATION, THE CAPTURE BUFFER IS NOT USED.
	CMP	CAPFLAG_,0
	JE	SAMPINTA
	JMP	SAMPINTC
;NOT CAPTURE, BUT RATHER, DMA SAMPLE
SAMPINTA:

;LOAD BUFFER POINTER (FOR COLLECTING THIS SAMPLE) INTO DX
	MOV	DX,SBIPTR_

;ADJUST INPUT POINTER FOR NEXT TIME
	MOV	AX,DX
	ADD	AX,SBINC_	;INCREMENT IT
	CMP	AX,SBEPTR_	;IS IT TOO BIG?
	JNE	SMPI0
;    POINTER MUST WRAP
	MOV	AX,SBBPTR_
;    POINTER IS NOW OKAY, SO STORE IT
SMPI0:	MOV	SBIPTR_,AX

;ADJUST BUFFER COUNT
	MOV	AX,SBCNT_	;GET COUNT
	INC	AX		;INCREMENT IT
	CMP	AX,SBMXCNT_	;COMPARE TO MAX
	JNE	SMPI1		;JUMP IF NO PROBLEM

   ;COUNT TOO HI, SO, ADVANCE OUTPUT POINTER
	MOV	AX,SBOPTR_	;GET POINTER IN AX
	ADD	AX,SBINC_	;ADJUST IT
	CMP	AX,SBEPTR_	;CMPARE WITH MAX+1
	JNE	SMPI2		;JUMP IF OKAY
	MOV	AX,SBBPTR_	;NOT OKAY, SO WRAP
SMPI2:	MOV	SBOPTR_,AX	; NOW STORE POINTER
	JMP	SMPI3		; DO NOT CHANGE BUFFER COUNT

SMPI1:	MOV	SBCNT_,AX	

SMPI3:	CLD

	GETDATA	0	;ASSUMES DX IS BASE PTR, USES DI,SI
	GETDATA	1
	GETDATA	2
	GETDATA	3
	GETDATA	4
	GETDATA	5
	GETDATA	6
	GETDATA	7
	GETDATA	8
	GETDATA	9
	GETDATA	A
	GETDATA	B
	GETDATA	C
	GETDATA	D
	GETDATA	E
	GETDATA	F	

SMPI4:

SMPIX:


	POP	DI
	POP	SI
	POP	DX
	POP	AX
	IRET

; GET DATA INTO CAPTURE BUFFER, AND DECREMENT CAPTURE BUFFER COUNT
SAMPINTC:
	DEC	CAPFLAG_

SAMPINTE:
	MOV	DX,OFFSET CAPDATA_
	JMP	SMPI3


;***********************************************************
;
;	BEGINNING OF MILLISECOND INTERRUPT ROUTINE
;		THE FILES SERIN0,SERIN8 AND SERINX MUST BE
;		LINK RIGHT AFTER DIGIREC, TO COMPLETE THE
;		MSINT CODE PROPERLY
;
;***********************************************************
MSINT:	PUSH	AX
	INTON
;SET MILLISECOND FLAG
	MOV	AX,0FFFFH
	MOV	msflag_,AX

;ADJUST SUB-MICROSECOND COUNTER
; DECIDE WHICH COUNT FOR NEXT MILLISECOND
	MOV	AX,CNTINC_
	ADD	REMDCNT,AX
	JNC	MSINTL
	MOV	AX,BASECNT1_
	JMP	CHGCNT

MSINTL:	MOV	AX,BASECNT0_
CHGCNT:
	;SEND OUT NEW COUNT, FIRST LOW BYTE, THEN HIGH
	OUT	CLK0,AL
	MOV	AL,AH
	OUT	CLK0,AL

MSINT3:
;CHECK TO SEE IF A SAMPLE INTERVAL IS OVER (MCOUNT_ BECOMES 1)
;DECREMENT THE MILLISECOND COUNTER
	DEC	MCOUNT_
	JE	MSINT1
	CMP	MCOUNT_,1
	JNE	MSINT2
	MOV	AL,0FFH
	OUT	DUART7+SET,AL
	JMP	MSINT2

MSINT1:
	MOV	AX,PERIOD_	
	MOV	MCOUNT_,AX
	MOV	AL,0FFH
	OUT	DUART7+RESET,AL

MSINT2:
;MAKE SURE A CONCURRENT UART SERVICE IS NOT IN PROGRESS
	TEST	MSINTFLG,0FFH
	JE	MSINT4
	POP	AX
	IRET
MSINT4:
; CHECK UARTS FOR INPUT
	PUSH	BX
	PUSH	BP
	PUSH	CX
	MOV	MSINTFLG,0FFH
	MOV	BP,DMAPTR_
	MOV	CX,DMACNT_

;**********************************************************
;
;   END OF CODE IN THIS FILE
;	MSINT CODE CONTINUES IN SERIN0.MAC, SERIN8.MAC AND SERINX.MAC
;
;**********************************************************


;*****************************************
;	DEFINE UART DATA STRUCTURES
;
	UDAT	0
	UDAT	1
	UDAT	2
	UDAT	3
	UDAT	4
	UDAT	5
	UDAT	6
	UDAT	7
	UDAT	8
	UDAT	9
	UDAT	A
	UDAT	B
	UDAT	C
	UDAT	D
	UDAT	E
	UDAT	F

;____________________________________________________________________
;____________________________________________________________________
;
;/////////////////////////   DATA SPACE   \\\\\\\\\\\\\\\\\\\\\\\\\\\
;____________________________________________________________________
;____________________________________________________________________

DATASEG	SEGMENT
	BSS	INTMASK:BYTE,1

	PUBLIC	MCOUNT_
	BSS	MCOUNT_:WORD,2
	BSS	SAMPFLG:BYTE,1
	BSS	DUMMY1:BYTE,1
	BSS	DUMMY2:BYTE,1

	PUBLIC	BASECNT0_
	PUBLIC	BASECNT1_
	PUBLIC	CNTINC_
			;THE PROCESSOR CLOCK IS A NOMINAL 14.7456
			; THIS IS DIVIDED BY 4, BEFORE REACHING THE
			; COUNTER THAT PRODUCES THE MILLISECOND INTERRUPT
			;THE MILLISECOND INTERRUPT COUNTER IS LOADED WITH
			; ONE OF TWO CONSTANTS, EACH MILLISECOND, IN ORDER
			; TO PRODUCE AN AVERAGE MILLISECOND INTERVAL OF
			; HIGH PRECISION.
			; BASECNT1_ AND BASECNT2_ ARE THE TWO CONSTANTS THAT
			; ARE USED.  CNTINC_ IS THE INCREMENT THAT IS
			; USED TO DYNAMICALLY DECIDE WHICH OF TWO CONSTANTS
			; SHOULD BE USED NEXT
	BSS	BASECNT0_:WORD,2
	BSS	BASECNT1_:WORD,2
	BSS	REMDCNT:WORD,2
	BSS	CNTINC_:WORD,2
	PUBLIC	PERIOD_
	BSS	PERIOD_:WORD,2
	PUBLIC	MSINTFLG		;MSINTFLG IS USED TO STOP THE
					; MSINT ROUTINE FROM SERVICING UARTS
					; IF IT IS ALREADY DOING IT.  E.G.
					; A SECOND MILLISECOND INTERRUPT CAN
					; COME IN, EVEN IF UARTS ARE STILL
					; BEING SERVICED FROM AN EARLIER INT.
					; THIS FLAG MAKES IT POSSIBLE TO
					; SERVICE THE MS COUNTER, WITHOUT
					; RE ENTERING THE UART SERVICE CODE.
	BSS	MSINTFLG:BYTE,2
	PUBLIC	CAPFLAG_
	BSS	CAPFLAG_:WORD,2
	PUBLIC	CAPDATA_
	BSS	CAPDATA_:WORD,256
	PUBLIC	SBMXCNT_
	PUBLIC	SBINC_
	PUBLIC	SBEPTR_
	PUBLIC	SBBPTR_
	PUBLIC	SBOPTR_
	PUBLIC	SBIPTR_
	PUBLIC	SBCNT_
	PUBLIC	DMAMXCNT_
	PUBLIC	DMACNT_
	PUBLIC	DMAPTR_
	BSS	DMAPTR_:WORD,2
	BSS	DMACNT_:WORD,2
	BSS	DMAMXCNT_:WORD,2
	BSS	SBCNT_:WORD,2
	BSS	SBIPTR_:WORD,2
	BSS	SBOPTR_:WORD,2
	BSS	SBBPTR_:WORD,2
	BSS	SBEPTR_:WORD,2
	BSS	SBINC_:WORD,2
	BSS	SBMXCNT_:WORD,2
	PUBLIC	msflag_
	BSS	msflag_:WORD,2

BUFFERSIZE_ EQU	06000H
	PUBLIC	BUFFER_
	BSS	BUFFER_:WORD,BUFFERSIZE_
DATASEG	ENDS
;****************************************************************
codeseg ends
	end	$begin

