/*
	DIGIREC.C
	PC DIGITAL DATA RECEIVER Program
 	DESCRIPTION: 80C88 "C" LANGUAGE MODULE FOR USE WITH DIGIREC.ASM,
				SERIN0.ASM,SERIN8.ASM AND SERINX.ASM
	APRIL 30, 1991
	COPYRIGHT CUTLER DIGITAL DESIGN 1991
	Rev.1  04-30-91 Reese T. Cutler << Original program >>
*/

#define version 1

/*	This program has been written to operate in a custom
	NEC V40 computer controller has 16 serial input ports and
	which has a 16 bit interface to the standard pc bus.

	The program calls on various assembly language routines which can be
	found in the assembly language file DIGIREC.ASM and which are described
	below.  In addition, some "C" language routines in this program
	can be called by assembly interrupt routines in DIGIREC.ASM

	The DIGIREC.ASM code (with SERIN0.ASM, SERIN8.ASM and SERINX.ASM)
	has all the hooks necessary for the application
	to work in a standalone	romable manner.

*/


/*
	The following assembly language routines are available:

	IOin(ioaddress)		 returns a byte value in an integer
	int ioaddress;		 corresponding to io byte register at
				 address "ioaddress"

	IOout(ioaddress,data)	loads byte value from "data" into io byte
	int 	ioaddress;	 register at address "ioaddress"
	int	data;

	INTON()			enables interrupts

	INTOFF()		disables interrupts

	ColdReset()		restarts the receiver board

	DISABLEU0()		DISABLES UART (DISABLED) WITH CUR PARAMS
	DISABLEU1()
	DISABLEU2()
	DISABLEU3()
	DISABLEU4()
	DISABLEU5()
	DISABLEU6()
	DISABLEU7()
	DISABLEU8()
	DISABLEU9()
	DISABLEUA()
	DISABLEUB()
	DISABLEUC()
	DISABLEUD()
	DISABLEUE()
	DISABLEUF()

	ENABLEU0()		ENABLES UART WITH CUR PARAMS
	ENABLEU1()
	ENABLEU2()
	ENABLEU3()
	ENABLEU4()
	ENABLEU5()
	ENABLEU6()
	ENABLEU7()
	ENABLEU8()
	ENABLEU9()
	ENABLEUA()
	ENABLEUB()
	ENABLEUC()
	ENABLEUD()
	ENABLEUE()
	ENABLEUF()

	DODMA()			SEND ANOTHER WORD BY DMA IF APPROPRIATE

	GetPoint()		GET THE POINTER TO BE BEGINNING OF THE DMA
				BUFFER SPACE.

*/


/* Command interpreter commands */
/* The PC invokes commands by writing a new (and different) command word.

   Results are returned in the status word (same I/O address as
   the command word).

   The high order byte of the command word is called the command byte,
   the low order byte is called the parameter byte.  The parameter byte must
   be changed before, or at the same time as the command byte.

   Upon completion, the command byte is placed in the high order byte of
   the status word, and varying values in the low order byte of the status
   word as specified below.

   The PC knows that its command has been accepted, when it sees
   the current command byte appear in the high order byte of the status word.

   The "nop" command is provided to allow spacing between repeat usage
   of the same command.

   The "general" command pulls together commands that have no parameter
   and no return value.  The parameter byte is used as a sub command.

*/

#include	"drcom.h"




#define	WORDS_IN_BUFFER	0x3000


/***  LOGICAL DEFINITIONS  ***/
#define true	1
#define false	0
#define SETREG	15
#define RESETREG	14
#define INREG	13

#define port0	0
#define port1	16
#define	INT_PORT	16
	#define	LOAD_IRQ	1
	#define	PCIE		2
	#define	INT_SEL_BITS	0x1C
#define port2	32
#define port3	48
#define NVRAM_PORT 48
#define port4	64
#define port5	80
#define	DMA_PORT	80
	#define	DMA_BIT	4
#define port6	96
#define port7	112

#define	LoDmaAdd	192
#define HiDmaAdd	193

#define LoConAdd	224
#define HiConAdd	240


struct uartdata
	{
	int	state;
	int	start_char;
	int	ststate;		/* internal, do not use */
	int	empbuf[4];
	int	shb0[4];
	int	sbh1[4];
	int	sbh2[4];
	int	bufcount;		/* internal, do not use */
	int	empptr;
	int	curptr;
	int	prevptr;		/* internal, do not use */
	int	baud_rate;
	int	parity;
	int	first_on;
	int	second_on;
	int	third_on;
	int	fourth_on;
	int	first_offset;
	int	second_offset;
	int	third_offset;
	int	fourth_offset;
	};

extern struct uartdata STATE0[16];

extern int	PERIOD;
extern int	BASECNT0;
extern int	BASECNT1;
extern int	CNTINC;
extern	int	CAPFLAG;
extern	int	CAPDATA[128];
extern	int	BUFFER[WORDS_IN_BUFFER];
extern	int	SBCNT;
extern	int	SBIPTR;
extern	int	SBOPTR;
extern	int	SBBPTR;
extern	int	SBEPTR;
extern	int	SBINC;
extern	int	SBMXCNT;
extern	int	DMACNT;
extern	int	DMAMXCNT;
extern	int	DMAPTR;
extern	int	msflag;

main()   /************************************************************/
{
int	Command, Subcommand, Parameter, Result;
int	CommandMem, SubcommandMem;
unsigned int	byte1,byte2,byte3,byte4,byte5,byte6;
int	capturebuf[64];
int	i,j,k;
int off,chn,urt;
int	interrupt;
int	nvr,sb0,sb1,sb2,recog0,recog1,recog2;

/* MAKE THINKS LOOK LIKE A RESET JUST TOOK PLACE */
IOout(LoConAdd,RESET);
IOout(HiConAdd,GENERAL);

/* WAIT FOR NOP COMMAND */
while ((CommandMem=IOin(HiConAdd)) != NOP);

/*input the parameter byte */
Parameter = IOin(LoConAdd)&0xff;

/* reflect the new command and parameter at output */
IOout(LoConAdd,Parameter);
IOout(HiConAdd,CommandMem);

while (true)
	{
	if((Command=IOin(HiConAdd)) != CommandMem)
		{
		CommandMem=Command;
		Parameter=IOin(LoConAdd)&0xff;
		Subcommand=Parameter;
		Result=Parameter;
		switch (Command)
			{
			case NOP:
				Result=0;
				break;
			case GENERAL:
				switch(Subcommand)
					{
					case CAPTURE:
						i=(byte2<<8)+byte1;
						CAPFLAG=i;
						break;
					case RESET:
						ColdReset();
						break;
					case DISABLE_INT:
						IOout(INT_PORT+RESETREG,PCIE);
						IOout(INT_PORT+SETREG,LOAD_IRQ);
						IOout(INT_PORT+RESETREG,LOAD_IRQ);
						break;
					case ENABLE_INT:
						IOout(INT_PORT+SETREG,PCIE);
						break;
					default:
						break;
					}
				break;
			case SHIFT_IN_BYTE:
				byte6=byte5;
				byte5=byte4;
				byte4=byte3;
				byte3=byte2;
				byte2=byte1;
				byte1=Parameter;
				Result=Parameter;
				break;
			case GET_NVR:
				Result=nvr;
				break;
			case READ_NVR:
				Result=read_nvram( ( (byte2<<8) | byte1), &nvr);
				break;
			case WRITE_NVR:
				Result=write_nvram( ( (byte2<<8) | byte1), Parameter);
				break;
			case SET_BAUD_RATE:
				Parameter=Parameter & 15;
				switch(byte1)
					{
					case B300:
						STATE0[Parameter].baud_rate=B300_VAL;
						break;
					case B600:
						STATE0[Parameter].baud_rate=B600_VAL;
						break;
					case B1200:
						STATE0[Parameter].baud_rate=B1200_VAL;
						break;
					case B2400:
						STATE0[Parameter].baud_rate=B2400_VAL;
						break;
					case B4800:
						STATE0[Parameter].baud_rate=B4800_VAL;
						break;
					case B9600:
						STATE0[Parameter].baud_rate=B9600_VAL;
						break;
					default:
						break;
					}
				Enable_Uart(Parameter);
				Result=Parameter;
				break;
			case SET_SYNC:
				Parameter=Parameter & 15;
				STATE0[Parameter].start_char=byte1;
				Result=Parameter;
				break;
			case SET_CLOCK:

				reset_clock((byte1+(byte2<<8)),
					    (byte3+(byte4<<8)),
					    (byte5+(byte6<<8)));
				Result=Parameter;
				break;
			case GET_BAUD_RATE:
				Parameter=Parameter & 15;
				i=STATE0[Parameter].baud_rate;
				switch(i)
					{
					case B300_VAL:
						Result=B300_VAL;
						break;
					case B600_VAL:
						Result=B600_VAL;
						break;
					case B1200_VAL:
						Result=B1200_VAL;
						break;
					case B2400_VAL:
						Result=B2400_VAL;
						break;
					case B4800_VAL:
						Result=B4800_VAL;
						break;
					case B9600_VAL:
						Result=B9600_VAL;
						break;
					default:
						Result=BBAD;
						break;
					}
				break;
			case SET_PARITY:
				Parameter=Parameter & 15;
				switch(byte1)
					{
					case NO_PARITY:
						STATE0[Parameter].parity=NO_PARITY_VAL;
						break;
					case B600:
						STATE0[Parameter].parity=EVEN_PARITY_VAL;
						break;
					case B1200:
						STATE0[Parameter].parity=ODD_PARITY_VAL;
						break;
					default:
						break;
					}
				Enable_Uart(Parameter);
				Result=Parameter;
				break;
			case GET_PARITY:
				Parameter=Parameter & 15;
				i=STATE0[Parameter].parity;
				switch(i)
					{
					case NO_PARITY_VAL:
						Result=NO_PARITY;
						break;
					case EVEN_PARITY_VAL:
						Result=EVEN_PARITY;
						break;
					case ODD_PARITY_VAL:
						Result=ODD_PARITY;
						break;
					default:
						Result=BAD_PARITY;
						break;
					}
				break;
			case DISABLE_UART:
				Parameter=Parameter & 15;
				Disable_Uart(Parameter);
				break;
			case GET_CAPTURE:
				while (CAPFLAG != 0) ;
				i=Parameter/2;
				if ((Parameter & 1)==1)
					Result=CAPDATA[i]>>8;
				else
					Result=CAPDATA[i];
				break;
			case SET_POSITION:
				/* Parameter has the channel # (0-63)
				   byte1 has the offset(0-63,any other then no data) */
				urt=(Parameter>>2)&0xf;
				chn=Parameter&3;
				if (byte1>63)
					off=128;
				else
					off=(byte1<<1);
				switch(chn)
					{
					case 0:
						STATE0[urt].first_offset=off;
						break;
					case 1:
						STATE0[urt].second_offset=off;
						break;
					case 2:
						STATE0[urt].third_offset=off;
						break;
					case 3:
						STATE0[urt].fourth_offset=off;
						break;
					}
				break;
			case GET_POSITION:
				/* Parameter has the channel # (0-63) */
				urt=(Parameter>>2)&0xf;
				chn=Parameter&3;
				switch(chn)
					{
					case 0:
						Result=STATE0[urt].first_offset;
						break;
					case 1:
						Result=STATE0[urt].second_offset;
						break;
					case 2:
						Result=STATE0[urt].third_offset;
						break;
					case 3:
						Result=STATE0[urt].fourth_offset;
						break;
					}
				break;
			case SET_INT:
				/* Parameter has the intnumber */
				switch(Parameter)
					{
					case INT_10:
						IOout(INT_PORT+RESETREG,INT_SEL_BITS); /*clear bits */
						IOout(INT_PORT+SETREG,(INT_10_VAL*4));
						Result=INT_10;
						break;
					case INT_15:
						IOout(INT_PORT+RESETREG,INT_SEL_BITS); /*clear bits */
						IOout(INT_PORT+SETREG,(INT_15_VAL*4));
						Result=INT_15;
						break;
					case INT_3:
						IOout(INT_PORT+RESETREG,INT_SEL_BITS); /*clear bits */
						IOout(INT_PORT+SETREG,(INT_3_VAL*4));
						Result=INT_3;
						break;
					case INT_5:
						IOout(INT_PORT+RESETREG,INT_SEL_BITS); /*clear bits */
						IOout(INT_PORT+SETREG,(INT_5_VAL*4));
						Result=INT_5;
						break;
					case INT_7:
						IOout(INT_PORT+RESETREG,INT_SEL_BITS); /*clear bits */
						IOout(INT_PORT+SETREG,(INT_7_VAL*4));
						Result=INT_7;
						break;
					default:
						IOout(INT_PORT+RESETREG,INT_SEL_BITS); /*clear bits */
						Result=NO_INT;
						break;
					}
				break;
			case SET_DMA:
				DMACNT=-1;  /* FREEZE DMA OPERATION */
				Parameter=Parameter & 3; /*strip off hi bits*/
				IOout(DMA_PORT+RESETREG,7);
				IOout(HiDmaAdd,0);
				switch(Parameter)
					{
					case ENABLE_DMA_CHANNEL_7:
						IOout(DMA_PORT+SETREG,7);
						Result=reset_buffer();
						break;
					case ENABLE_DMA_CHANNEL_6:
						IOout(DMA_PORT+SETREG,6);
						Result=reset_buffer();
						break;
					case ENABLE_DMA_CHANNEL_5:
						IOout(DMA_PORT+SETREG,5);
						Result=reset_buffer();
						break;
					case DISABLE_DMA:
					default:
						Result=get_count();
						break;
					}
				break;
			default:
				break;
			}
		IOout(LoConAdd,Result);
		IOout(HiConAdd,Command);
		}

	DODMA();

	}
} /*******  End main() ***********************************************/

reset_clock(lowcount,hicount,increment)
int lowcount;
int hicount;
int increment;
{
INTOFF();

BASECNT0=lowcount;
BASECNT1=hicount;
CNTINC=increment;

INTON();
}


reset_buffer()
{
int buffer_count;
int i,j,len,bcplus,len2;
int ptr;

len=get_count();

buffer_count=((WORDS_IN_BUFFER-64)/len);
bcplus=buffer_count+1;
ptr = GetPoint() + (len * 2 * buffer_count);
len2= len *2;

INTOFF();

SBCNT=0;
DMAPTR=SBIPTR=SBOPTR=SBBPTR=GetPoint();
SBEPTR=ptr;
SBMXCNT=bcplus;
SBINC=len2;
DMAMXCNT=len;
DMACNT=0;

INTON();

return(len);

}

get_count()
{
int i,j,len;

len=0;
for (i=0;i<16;i++)   /* for each of 16 uarts */
	{
	j=STATE0[i].first_offset;
	if ((j!=128) && (len<j)) len=j;
	j=STATE0[i].second_offset;
	if ((j!=128) && (len<j)) len=j;
	j=STATE0[i].third_offset;
	if ((j!=128) && (len<j)) len=j;
	j=STATE0[i].fourth_offset;
	if ((j!=128) && (len<j)) len=j;
	}
len=(len>>1)+1;
if (len<1) len=1;
if (len>64) len=64;
return(len);
}


/*******************************************************************
*
*	Enable_Uart()
*
*******************************************************************/
Enable_Uart(p)
int p;
{
switch(p)
	{
	case 0:
		ENABLEU0();
		break;
	case 1:
		ENABLEU1();
		break;
	case 2:
		ENABLEU2();
		break;
	case 3:
		ENABLEU3();
		break;
	case 4:
		ENABLEU4();
		break;
	case 5:
		ENABLEU5();
		break;
	case 6:
		ENABLEU6();
		break;
	case 7:
		ENABLEU7();
		break;
	case 8:
		ENABLEU8();
		break;
	case 9:
		ENABLEU9();
		break;
	case 10:
		ENABLEUA();
		break;
	case 11:
		ENABLEUB();
		break;
	case 12:
		ENABLEUC();
		break;
	case 13:
		ENABLEUD();
		break;
	case 14:
		ENABLEUE();
		break;
	case 15:
		ENABLEUF();
		break;
	default:
		break;
	}
}

/*******************************************************************
*
*	Disable_Uart()
*
*******************************************************************/
Disable_Uart(p)
int p;
{
switch(p)
	{
	case 0:
		DISABLEU0();
		break;
	case 1:
		DISABLEU1();
		break;
	case 2:
		DISABLEU2();
		break;
	case 3:
		DISABLEU3();
		break;
	case 4:
		DISABLEU4();
		break;
	case 5:
		DISABLEU5();
		break;
	case 6:
		DISABLEU6();
		break;
	case 7:
		DISABLEU7();
		break;
	case 8:
		DISABLEU8();
		break;
	case 9:
		DISABLEU9();
		break;
	case 10:
		DISABLEUA();
		break;
	case 11:
		DISABLEUB();
		break;
	case 12:
		DISABLEUC();
		break;
	case 13:
		DISABLEUD();
		break;
	case 14:
		DISABLEUE();
		break;
	case 15:
		DISABLEUF();
		break;
	default:
		break;
	}
}

/*********************************************************************
 *
 *			read_nvram
 *
 *********************************************************************
 */
read_nvram(address,data)
int address;
int *data;
{
int	temp;
int 	temp1;

IOout(port2+RESETREG,0xff);
IOout(port2+SETREG,address);

/* set up dummy write, to load address */

/* remove for lower density nvram
   if(nvr_send_start(0,address) == 0)
		{
		nvr_stop();
		return(4);
		}

   if(nvr_send_byte(address) != 0)
		{
		nvr_stop();
		return(5);
		}

*/

/* set up read */
   if(nvr_send_start(1,address) == 0)
		{
		nvr_stop();
		return(6);
		}
   *data=nvr_read_byte();

/* conclude interaction */
   nvr_stop();

IOout(port4+RESETREG,0xff);
IOout(port4+SETREG,*data);

/* and exit */
   return(0);
}

/*********************************************************************
 *
 *			write_nvram
 *
 *********************************************************************
 */
write_nvram(address,data)
int address;
{
int	temp;
int 	temp1;

/* set up write, with full address */
   if(nvr_send_start(0,address) == 0)
		{
		nvr_stop();
		return(1);
		}

/* remove for lower density nvram
   if(nvr_send_byte(address) != 0)
		{
		nvr_stop();
		return(2);
		}
*/

/* now write data */
   if(nvr_send_byte(data) != 0)
		{
		nvr_stop();
		return(3);
		}

/* conclude interaction */
   nvr_stop();

/* and exit */
   return(0);
}

nvr_send_start(rw,address)
int rw;
int address;
{
int	ack,slab;
int mscount;

IOout(port0+SETREG,0xff);
IOout(port0+RESETREG,0xff);
mscount=0;

INTOFF();
/* make slave address byte */
/* adjust for lower density nvram
slab= 0xa0 | ( (address>>7) & 0xe );
*/
slab= (address << 1) & 126;

if (rw!=0) slab |= 1 ;

/*send successful read request */
do
	{
	/* issue start */
	nvr_start();

	/* send byte, and get acknowledge bit */
	ack=nvr_send_byte(slab);

	/* do time check for timeout */
	mscount=mscount+get_tick();
	}
while((mscount<20250) && (ack!=0));

INTON();
/* if timeout condition, the return 0 as error flag */
if (mscount == 20250) return(0);
return(1);

}

get_tick()
{
int temp;
/* this routine may miss ticks, but will not produce any
	false ticks */
if (msflag!=0)
	{
	msflag=0;
	return(1);
	}
return(0);
}

nvr_send_byte(address)
int address;
{
nvr_send_bit(address & 0x80);
nvr_send_bit(address & 0x40);
nvr_send_bit(address & 0x20);
nvr_send_bit(address & 0x10);
nvr_send_bit(address & 0x08);
nvr_send_bit(address & 0x04);
nvr_send_bit(address & 0x02);
nvr_send_bit(address & 0x01);

return(nvr_get_bit());
}

nvr_read_byte()
{
int i,temp;

temp=0;
for(i=0;i<8; i++)
	{
	temp = (temp<<1) | nvr_get_bit();
	}
nvr_send_bit(1);	/* skip acknowledge */
return(temp);
}

nvr_get_bit()
{
int temp;
IOout(NVRAM_PORT+RESETREG,2); /*free up data line */

IOout(NVRAM_PORT+SETREG,4);	/* turn on clock */

temp=IOin(NVRAM_PORT+INREG);	/* read data line */

IOout(NVRAM_PORT+RESETREG,4);	/* turn off clock */

if ( (temp& 1)!=0) return(1); else return(0);
}

nvr_send_bit(bit)
int bit;
{
int temp;

if(bit==0)
	IOout(NVRAM_PORT+SETREG,2);
else
	IOout(NVRAM_PORT+RESETREG,2);

IOout(NVRAM_PORT+SETREG,4);	/* turn on clock */

/* the following line is just for a pause, may not be necessary */
temp=IOin(NVRAM_PORT+INREG);	/* read data line */

IOout(NVRAM_PORT+RESETREG,4);	/* turn off clock */
}

nvr_start()
{
IOout(NVRAM_PORT+RESETREG,1); /*be sure datum to gate is low */

IOout(NVRAM_PORT+RESETREG,4); /* be sure clock is low */

IOout(NVRAM_PORT+RESETREG,2);	/* data high, by disabling tristate driver */

IOout(NVRAM_PORT+SETREG,4);	/* clock high */

IOout(NVRAM_PORT+SETREG,2);	/* data lo */

IOout(NVRAM_PORT+RESETREG,4);	/* clock lo */

}

nvr_stop()
{
IOout(NVRAM_PORT+RESETREG,1); /*be sure datum to gate is low */

IOout(NVRAM_PORT+RESETREG,4); /* be sure clock is low */

IOout(NVRAM_PORT+SETREG,2);	/* be sure datum is low (enable tristate) */

IOout(NVRAM_PORT+SETREG,4);	/* clock high */

IOout(NVRAM_PORT+RESETREG,2);	/* datum hi */

IOout(NVRAM_PORT+RESETREG,4);	/* clock lo */

}

