/*
	DRCOM.H
	Defines for the different commands issued by a PC like
	computer through the io port (usually 300) to control the
	digital data receiver

	APRIL 30, 1991
	COPYRIGHT CUTLER DIGITAL DESIGN 1991
*/



/* Command interpreter commands */
/* The PC invokes commands by writing a new (and different) command word.

   Results are returned in the status word (same I/O address as
   the command word).

   The high order byte of the command word is called the command byte,
   the low order byte is called the parameter byte.  The parameter byte must
   be changed before, or at the same time as the command byte.

   Upon completion, the command byte is placed in the high order byte of
   the status word, and varying values in the low order byte of the status
   word as specified below.

   The PC knows that its command has been accepted, when it sees
   the current command byte appear in the high order byte of the status word.

   The "nop" command is provided to allow spacing between repeat usage
   of the same command.

   The "general" command pulls together commands that have no parameter
   and no return value.  The parameter byte is used as a sub command.

*/

#define	NOP		0		/* dummy command, does nothing,
					   return = 0 */
#define	GENERAL		1		/* collection of commands that have
						no parameters,
					   return = subcommand */
	/* general sub commands */
	#define	RESET		0
	#define	CAPTURE		8	/* CAPTURE A DATA SAMPLE */
				/* 16 bit count in byte1(lo) and byte 2(hi) */
	#define	DISABLE_INT	9
	#define	ENABLE_INT	10

#define	SET_DMA		2
	#define	ENABLE_DMA_CHANNEL_7	3
	#define	ENABLE_DMA_CHANNEL_6	2
	#define	ENABLE_DMA_CHANNEL_5	1
	#define	DISABLE_DMA				0
						/* return=NUMBER OF DATA CHANNELS (1-64)*/

#define	SHIFT_IN_BYTE	3		/* auxilliary parameter shift regist
					   has 4 bytes, this command shifts
					   in a new low order byte.
					   return=parameter byte. */

#define	WRITE_NVR	4		/* write the byte in Parameter
					   into the nvram location indicated
					   by the address in byte shift
					   register, the return value for this
					   operation is 0 for unsuccessful,
					   non-zero, for successful */

#define	READ_NVR	5		/* read the byte
					   from the nvram location indicated
					   by the address in byte shift
					   register, and store it.
					   The return value for this
					   operation is 0 for unsuccessful,
					   non-zero, for successful.
					   Data that has been read is available
					   using the GET_NVR command */

#define	GET_NVR		6		/* return whatever is in nvr storage */

#define	SET_BAUD_RATE	9		/* set the baud rate of the uart
					   specified in parameter to the
					   rate specified in the auxiliary
					   shift register
					   return=parameter & 00001111B */

#define	GET_BAUD_RATE	10		/* get the baud rate of the uart
					   specified in parameter.
					   return=baud rate */

	#define	B300	0
	#define	B600	1
	#define	B1200	2
	#define	B2400	3
	#define	B4800	4
	#define	B9600	5
	#define	BBAD	6

	#define	B300_VAL	0X44
	#define	B600_VAL	0X55
	#define	B1200_VAL	0X66
	#define	B2400_VAL	0X88
	#define	B4800_VAL	0X99
	#define	B9600_VAL	0XBB



#define	SET_PARITY	11	/* channel in parameter, parity value in
				   byte1,   return= parameter */

#define	GET_PARITY	12	/* channel in parameter, return = parity */

	#define	NO_PARITY	0
	#define	EVEN_PARITY	1
	#define	ODD_PARITY	2
	#define	BAD_PARITY	3

	#define	NO_PARITY_VAL	19
	#define	EVEN_PARITY_VAL	3
	#define	ODD_PARITY_VAL	7


#define	DISABLE_UART	13
#define	SET_POSITION	14
#define	GET_POSITION	15

#define	SET_INT		16		/* parameter holds int choice
					   returns parameter */


	#define	INT_10		0
	#define	INT_15		1
	#define	INT_3		2
	#define	INT_5		3
	#define	INT_7		4
	#define	NO_INT		5

	#define	INT_10_VAL	5
	#define	INT_15_VAL	4
	#define	INT_3_VAL	3
	#define	INT_5_VAL	2
	#define	INT_7_VAL	1


#define	SET_SYNC	17
#define	SET_CLOCK	20
#define	GET_CAPTURE	100
