;********************************************************************
;
; FILE: mdigiasm.asm				(R. Cutler 04/30/91)
;
;	The following c callable routines in assembly language:
;
;	irq_entry()	interrupt routine to handle digirec interrupt
;			 does some interrupt housekeeping and calls
;			 the c routine "dr_isr"
;
;	input_word(int address)
;			routine in input a full word from an i/o port
;			 the address is normally in the range of 0-3FF
;			 the return value is the contents of the i/o port
;
;	outpu_word(int address, int data)
;			routinge to output a full word to an i/o port
;			 the address is normally in the range of 0-3FF
;			 the return value is indeterminate (currently
;			 it reflects the value of data written to i/o)
;
;	getxmem(long source_add, long destination_add, int word_count)
;			routine to transfer data words (16 bits) between
;			 memory locations, including extended memory
;
;	SysInt(int flag)
;			routine to set or clear the interrupt flag
;			 if flag is 0200H, interrupts are enabled
;			 if flag is 0000H, interrupts are disabled
;			 the state of the system interrupt flag, before
;			 being altered, is the return value for this routine.
;
;********************************************************************	

;define as medium model:
M	equ 0

		include model.inc

DGROUP	GROUP	_DATA
_TEXT		segment	byte public 'CODE'
		assume	cs:_TEXT,ds:_DATA

;****************************************************************
;
;		_irq_entry
;
;****************************************************************
       PUBLIC  _irq_entry

       EXTRN   _dr_isr:near

FAR_BSS	segment word
	extrn	_digirec:word
FAR_BSS	ends


		assume	cs:_TEXT,ds:DGROUP

_irq_entry proc far

	push	bp
	push	ax
	push	bx
	push	cx
	push	dx
	push	ds
	push	es
	push	di
	push	si
	push	sp

; enable other interrupts
	sti

; set up for far data
	push	es
	mov	ax,FAR_BSS
	mov	es,ax
	assume es:FAR_BSS


; restore ds
	pop	es

		assume	cs:_TEXT,ds:DGROUP
;inform both interrupt controllers
;
	mov	al,20h		;non specific eoi for both controllers
	out	20h,al		;send to first controller
	out	0a0h,al		;send to second controller

;insure that ds is pointing to the right segment
	mov	ax,DGROUP
	mov	ds,ax

; call c routine to handle data
	push	cs
	call	_dr_isr

; set up for far data
	mov	ax,FAR_BSS
	mov	es,ax
	assume es:FAR_BSS

;clear digirec interrupt
	mov	dx,es:_digirec	;base address of digirec
	inc	dx		;increment by 2 to interrupt off flag
	inc	dx
	in	ax,dx		;clear by reading, no valid data

; restore registers

	pop	sp
	pop	si
	pop	di
	pop	es
	pop	ds
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	pop	bp
       iret
_irq_entry endp


;****************************************************************
;
;		_input_word
;
;****************************************************************
		public	_input_word

_DATA		segment word
p1		struc				; stack structure
		dw	?			; old bp
		dw	return_size dup (?)	; return address
iaddress	dw	?			;input address
p1		ends
_DATA		ends

		assume	cs:_TEXT,ds:DGROUP
PROCEDURE	_input_word
		push	bp			; save registers
		mov	bp,sp
		push	dx

		mov	dx,[bp].iaddress	;get address
		in	ax,dx

		pop	dx			; restore registers
		pop	bp
		ret
_input_word		endp

;****************************************************************
;
;		_output_word
;
;****************************************************************
		public	_output_word

_DATA		segment word
p2		struc				; stack structure
		dw	?			; old bp
		dw	return_size dup (?)	; return address
oaddress	dw	?			;output address
data		dw	?			;data
p2		ends
_DATA		ends

		assume	cs:_TEXT,ds:DGROUP
PROCEDURE	_output_word
		push	bp			; save registers
		mov	bp,sp
		push	dx

		mov	dx,[bp].oaddress	;get address
		mov	ax,[bp].data		;get data
		out	dx,ax

		pop	dx			; restore registers
		pop	bp
		ret
_output_word		endp

;****************************************************************
;
;		_getxmem
;
;****************************************************************
		public	_getxmem

_DATA		segment word
p3		struc				;stack structure
		dw	?			;old bp
		dw	return_size dup (?)	;return address
souradd		dw	?
souraddhi	dw	?
destadd		dw	?
destaddhi	dw	?
wdcnt		dw	?
p3		ends

p4		struc
dummy		dw	8 dup (?)
sptr		dw	?
slo		dw	?
shi		db	?
sac		db	?
sresv		dw	?
dptr		dw	?
dlo		dw	?
dhi		db	?
dac		db	?
p4		ends


_mygdt		dw	48 dup (?)

_DATA		ends

		assume	cs:_TEXT,ds:DGROUP
PROCEDURE	_getxmem
		push	bp			; save registers
		mov	bp,sp
		push	ds
		push	es
		push	bx
		push	cx
		push	si
		push	di
; set up extra segment
		mov	ax,seg _mygdt
		mov	es,ax

; zero dummy
		mov	ax,0
		mov	bx,offset _mygdt
		mov	es:[bx].dummy,ax
		mov	es:[bx].dummy+2,ax
		mov	es:[bx].dummy+4,ax
		mov	es:[bx].dummy+6,ax

; load memory segment - 1 and access code
		mov	ax,0ffffh
		mov	es:[bx].sptr,ax
		mov	es:[bx].dptr,ax
		mov	al,093h
		mov	es:[bx].sac,al
		mov	es:[bx].dac,al

; load source address
		mov	ax,[bp].souradd
		mov	es:[bx].slo,ax
		mov	ax,[bp].souraddhi
		mov	es:[bx].shi,al

; load destination address
		mov	ax,[bp].destadd
		mov	es:[bx].dlo,ax
		mov	ax,[bp].destaddhi
		mov	es:[bx].dhi,al

; set up source index
		mov	si,bx

; set up word count
		mov	cx,[bp].wdcnt

; set up command
		mov	ah,087h		;move block command

; issue interrupt 15
		int	015h

		mov	al,0
		xchg	al,ah
		
;pop and return
gtxmx:
		pop	di
		pop	si
		pop	cx
		pop	bx
		pop	es
		pop	ds
		pop	bp
		ret
_getxmem	endp

;****************************************************************
;
;		_SysInt
;
;****************************************************************
		PUBLIC	_SysInt

_DATA		segment word
SysIntp	struc				; stack structure
		dw	?			; old bp
		dw	return_size dup (?)	; return address
intflag		dw	?			;first word argument
		dw	?			;second word argument
SysIntp	ends


_DATA		ends

		assume	cs:_TEXT,ds:DGROUP
PROCEDURE	_SysInt
		push	bp
		mov	bp,sp
	
;save flags for use in setting return value
		pushf

		mov	ax,[bp].intflag
		or	ax,ax
		jne	si1
		cli
		jmp	si2
si1:		sti
si2:
; set return value to interrupt flag, before adjusting it
		pop	ax
		and	ax,0200h	

		pop	bp
		ret
_SysInt	endp

_TEXT		ends
		end

