/* FILE: mdigirec.h                             (R. Cutler 04/30/91)

This is an include file of the defines, data structure definitions and
external data declarations for using the mdigirec module.

*/

#ifndef _MDIGIREC_
#define _MDIGIREC_

/************************************************************************
                               INCLUDES

************************************************************************/
#include "mconst.h"
#include "mqueue.h"
#include "mdemux.h"


/************************************************************************
                                DEFINES

************************************************************************/
/* Constants */
#define	DR_OKAY			0

#define DIGIREC_INPUT_RANGE         5.00           /* in volts, 20 = +/- 10V */
#define DIGIREC_MID_POINT           2048            /* 10 bit */
#define	DIGIREC_COUNTS		4096.0
#define DIGIREC			4096		/*identifier for digirec board */

#define	DR_BASE_ADDRESS		0x0300
#define	DR_DMA_CHANNEL		6
#define	DR_INT_CHANNEL		10

#define BOARD_CLOCK_FREQUENCY   4e6
#define HIGH_BYTE_ENABLE        0x0001
#define MAX_CHANNEL             128
#define MAX_GAIN                8

/* Defaults */
#define CHANNEL_GAIN            4
#define CLOCK_SOURCE            INTERNAL_CLOCK
#define DIGITIZATION_RATE       100.0           /* Samples/sec/channel */
#define TIME_OUT                10
#define TRIGGER_SOURCE          INTERNAL_TRIGGER

/* Buffer Constants */
#define MAX_BUFFER_SIZE         16384L


/* bit definitions for the A/D timing source */

#define INTERNAL_CLOCK		0
#define EXTERNAL_CLOCK		1
#define INTERNAL_TRIGGER	0
#define EXTERNAL_TRIGGER	2

/*************************************************************************
 *			STRUCTURE DEFINITIONS
 ************************************************************************/

/************************************************************************
                         EXTERNAL DECLARATIONS

These functions can be called from all modules that include this file.

************************************************************************/
PUBLIC FLAG dr_buffer_full ();
PUBLIC void dr_dump_configuration (FILE *);
PUBLIC DM_INFO far * dr_get_collect_info ();
PUBLIC unsigned long dr_get_buffer_size ();
PUBLIC unsigned int  dr_get_channel_gain (unsigned int);
PUBLIC unsigned long dr_get_channel_size ();
PUBLIC char dr_get_clock_source ();
PUBLIC double dr_get_digitization_rate ();
PUBLIC void dr_get_new_buffers ();
PUBLIC Q_BUFFER far * dr_get_next_buffer ();
PUBLIC int dr_get_number_of_ext_buffers ();
PUBLIC int dr_get_scan_count ();
PUBLIC char dr_get_trigger_source ();
PUBLIC void dr_initialize_buffers ();
PUBLIC void dr_initialize_collection();
PUBLIC void dr_initialize_params();
PUBLIC void dr_set_channel (unsigned int, unsigned int);
PUBLIC void dr_set_ChannelBlocksize (unsigned long);
PUBLIC void dr_set_ChannelGain (unsigned int);
PUBLIC void dr_set_ClockSource (int);
PUBLIC void dr_set_DigitizationRate (double);
PUBLIC void dr_set_TriggerSource (int);
PUBLIC void dr_start_collection ();
PUBLIC void dr_stop_collection ();

#endif
