/* file: MDIGISUP.C                              (R. Cutler 04/30/91)
*/


/*************************************************************************
                             INCLUDE FILES

*************************************************************************/
#include	<stdio.h>
#include	<conio.h>
#include	<string.h>
#include	<ctype.h>
#include	<dos.h>
#include	<malloc.h>
#include	<time.h>

#include	"drcom.h"
#include	"mdigisup.h"
#include	"merror.h"
#include	"xdetect.h"

/*=======================================================================*
 *                           define                                      *
 *=======================================================================*/
/* local defines                                                         */
#define	DR_OKAY			1
#define	DMA_OKAY		0
#define	OKAY			0
#define	BAD			1
#define	FALSE			0
#define TRUE			1

/* INT 21 call parameters */
#define	GET_IRQ_VEC	0x35
#define	SET_IRQ_VEC	0x25

/* Interrupt controller constant and registers */
#define	INTA1	0x21
#define	INTB1	0xa1


/*=======================================================================*
 *                           function pre definitions                    *
 *=======================================================================*/
void far irq_entry();
void(far * ie_ptr)();


/*=======================================================================*
 *                             global variables                          *
 *=======================================================================*/

/* arrays for dynamic choice of interrupts and dma channels */
static int pagereg[8]={ 0x87, 0x83, 0x81, 0x00, 0x82, 0x8b, 0x89, 0x8a};
static int addr[8]   ={ 0x00, 0x02, 0x04, 0x06, 0xc0, 0xc4, 0xc8, 0xcc};
static int cntreg[8] ={ 0x01, 0x03, 0x05, 0x07, 0xc2, 0xc6, 0xca, 0xce};
static int chenb[8]  ={ 0, 1, 2, 3, 0, 1, 2, 3};
static int chdsb[8]  ={ 4, 5, 6, 7, 4, 5, 6, 7};
static int statmsk[8]={ 1, 2, 4, 8, 1, 2, 4, 8};
static int inmode[8] ={ 0x44, 0x45, 0x46, 0x47, 0x44, 0x45, 0x46, 0x47};
static int outmode[8]={ 0x48, 0x49, 0x4a, 0x4b, 0x48, 0x49, 0x4a, 0x4b};
static int statreg[8]={  0x8,  0x8,  0x8,  0x8, 0xd0, 0xd0, 0xd0, 0xd0};
static int maskreg[8]={  0xa,  0xa,  0xa,  0xa, 0xd4, 0xd4, 0xd4, 0xd4};
static int modereg[8]={  0xb,  0xb,  0xb,  0xb, 0xd6, 0xd6, 0xd6, 0xd6};
static int byteff[8] ={  0xc,  0xc,  0xc,  0xc, 0xd8, 0xd8, 0xd8, 0xd8};
static char irq[16]  ={ 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
			0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77 };
static char enable_irq[16]
		     ={ 0xfe, 0xfd, 0xfb, 0xf7, 0xef, 0xdf, 0xbf, 0x7f,
			0xfe, 0xfd, 0xfb, 0xf7, 0xef, 0xdf, 0xbf, 0x7f };
static int  ireg[16] ={INTA1,INTA1,INTA1,INTA1,INTA1,INTA1,INTA1,INTA1,
                       INTB1,INTB1,INTB1,INTB1,INTB1,INTB1,INTB1,INTB1};


static unsigned int	lastcomm;
unsigned int far	digirec;
static unsigned int	intchan=10;
static unsigned int	intflag=NO_INT;
static unsigned int	dmachan=6;
static	unsigned int	dmaflag=DISABLE_DMA;

union REGS	inregs,outregs;
struct SREGS	segregs;
unsigned	irqseg_sav,irqoff_sav;

/*=======================================================================*
 *			External Memory Variables
 *=======================================================================*/
PRIVATE unsigned long xm_lo_add,xm_hi_add;
PRIVATE	unsigned int xm_block_count, xm_pages_used;
PRIVATE	unsigned long _xmalloc(unsigned long byte_count);

PRIVATE	X_BUF far * ehead;
PRIVATE	X_BUF far * etail;
PRIVATE	X_BUF far * fhead;
PRIVATE	X_BUF far * ftail;
PRIVATE	X_BUF far * process;
PRIVATE	X_BUF far * collect;

PRIVATE	int	dma_err = DMA_OKAY;

/*=======================================================================*
 *                           collect_to_full
 *=======================================================================*/
PRIVATE
collect_to_full()
{
if (collect != NULL)
	{
	if (ftail==NULL)
		{
		fhead=collect;
		}
	else
		{
		ftail->next=collect;
		}
	ftail=collect;
	ftail->next=NULL;
	collect=NULL;
	return(TRUE);
	}
else
	return(FALSE);
}

/*=======================================================================*
 *                               compute_time_nums
 *=======================================================================*/
PRIVATE
int compute_time_nums(freq,lowl,lowh,highl,highh,incrementl,incrementh)
double freq;
int	*lowl, *lowh, *highl, *highh, *incrementl, *incrementh;
{
int	low;
int	high;
int	increment;
double r,s;

r=250.0 * freq;
low=(int)(r);
high=low+1;
s= r - (double) low ;
increment=(int)((65536.0*s)+0.5);
*lowl=low&255;
*lowh=low >> 8;
*highl=high&255;
*highh=high >> 8;
*incrementl=increment&255;
*incrementh=increment >> 8;

}

/*=======================================================================*
 *                               dr_128K_blocks_used
 *=======================================================================*/
PUBLIC
unsigned int dr_128K_blocks_used()
{
return(xm_pages_used);
}

/*=======================================================================*
 *                               dr_dma_status
 *=======================================================================*/
PUBLIC
int dr_dma_status()
{
int stat;

return( statmsk[dmachan] & inp(statreg[dmachan]) );

}

/*=======================================================================*
 *                                 dr_full_status
 *=======================================================================*/
PUBLIC
int	dr_full_status()
{

return(fhead!=NULL);
}

/*=======================================================================*
 *                                 dr_get_dma_buffer
 *=======================================================================*/
PUBLIC
unsigned long dr_get_dma_buffer()
{
int q;
int intflag;

/* wait for next buffer, return valid buffer address, or
	return 0 for over run or timeout */

while ((fhead==NULL))
	{
	if(dma_err!=DMA_OKAY) return(0L);
	}

/* if process is not NULL, then, transfer it to empty list before getting
   a new process buffer */
intflag=SysInt(0);	/* turn off interrupt */
process_to_empty();
SysInt(intflag);	/*restore interrupt */

intflag=SysInt(0);	/* turn off interrupt */
full_to_process();
SysInt(intflag);

return(process->pointer);
}

/*=======================================================================*
 *                                 dr_init_xm
 *=======================================================================*/
PUBLIC
int dr_init_xm(blk_cnt)
unsigned int blk_cnt;
{
union REGS irg,org;
unsigned long temp;

/* get # of 1K blocks of extended memory (beyond 1M) */
irg.h.ah=0x88;
xm_block_count=int86(0x15,&irg,&org);

if (xm_block_count<blk_cnt) er_abort(DR_EXT_MEM);

xm_lo_add=0x100000;
temp=(long)xm_block_count;
xm_hi_add=(temp * 1024)+xm_lo_add;
temp=(long)blk_cnt;
xm_lo_add=xm_hi_add - (temp * 1024);
xm_pages_used=0;
return(DR_OKAY);
}

/*=======================================================================*
 *		dr_init_xm_buff
 *=======================================================================*/
PUBLIC
unsigned int dr_init_xm_buff(bs)
unsigned long bs;
{
unsigned long xadd;
unsigned int count,c1;
PRIVATE	X_BUF far * temp;

ehead=NULL;
etail=NULL;
fhead=NULL;
ftail=NULL;
collect=NULL;
count = 0;

while ( (xadd=_xmalloc(bs*2)) != 0L)
	{
	if ((process = (X_BUF far *) _fmalloc(sizeof(X_BUF)))== NULL)
		er_abort(DR_INT_MEM);
	process->length=bs;
	process->pointer=xadd;
	count += 1;
	process_to_empty();
	}
process = NULL;

return(count);
}

/*=======================================================================*
 *                                 dr_irq_restore
 *=======================================================================*/
PUBLIC
void dr_irq_restore()
{
  unsigned i;

/* disable current interrupt */
    i = inp(ireg[intchan]);
    outp(ireg[intchan],(i | ~enable_irq[intchan]));

/* restore previous vector */
    inregs.h.ah = SET_IRQ_VEC;
    inregs.h.al = irq[intchan];
    inregs.x.dx = irqoff_sav;
    segregs.ds = irqseg_sav;
    intdosx(&inregs,&outregs,&segregs);
}

/*=======================================================================*
 *                                 dr_irq_setup
 *=======================================================================*/
PUBLIC
void dr_irq_setup()
{
  unsigned i;
/* set up to get existing irq vector */
    inregs.h.ah = GET_IRQ_VEC;
    inregs.h.al = irq[intchan];
/* do it */
    intdosx(&inregs,&outregs,&segregs);
/* save values */
    irqseg_sav = segregs.es;
    irqoff_sav = outregs.x.bx;

/* set up to load a new irq vector */
    ie_ptr = irq_entry;
    inregs.h.ah = SET_IRQ_VEC;
    inregs.h.al = irq[intchan];
    inregs.x.dx = FP_OFF(ie_ptr);
    segregs.ds = FP_SEG(ie_ptr);
/* do it */
    intdosx(&inregs,&outregs,&segregs);

/* enable new interrupt */
    i = inp(ireg[intchan]);
    outp(ireg[intchan],(i & enable_irq[intchan]));
}

/*=======================================================================*
 *                        dr_isr
 *=======================================================================*/
PUBLIC
int dr_isr()         /* IRQ serv routine called from irq_entry in irq.asm */
{
int i;

	/* SERVICE INTERRUPT */

/* move collect buffer to a full list */
if (collect_to_full())
	{
	if (dma_err == DMA_OKAY)
		dma_err = dr_start_dma();
	}
else
	{
	dma_err = DR_DMA_BLOCKED;
	}
}


/*=======================================================================*
 *                                 dr_issue_command
 *=======================================================================*/
PUBLIC
int dr_issue_command(comm,parm)
int	comm;
int	parm;
{
if (comm==lastcomm) dr_send_command( NOP,0);
return(dr_send_command(comm,parm));

}

/*=======================================================================*
 *                        dr_nvread
 *=======================================================================*/
PUBLIC
int dr_nvread(add,da)
int add;
int *da;
{
int status;
int lim;
int add1;

add1=255&(add>>8);
add=255&add;
lim=-1;
status=1;
while ((lim<10000) && (status != 0))
	{
	dr_issue_command(SHIFT_IN_BYTE,add1);
	dr_issue_command(SHIFT_IN_BYTE,add);
	status=dr_issue_command(READ_NVR,0);
	*da=dr_issue_command(GET_NVR,0);
	lim+=1;
	}
return(lim<10000);
}

/*=======================================================================*
 *                        dr_nvwrite
 *=======================================================================*/
PUBLIC
int dr_nvwrite(add,da)
int add;
int da;
{
int stat;
int rda;
int count=0;

da=da&255;
stat=nv_try_write(add,da);

while ((stat != 0) && (count<50))
	{
	stat=nv_try_write(add,da);
	count += 1;
	}
return(count<50);
}

/*=======================================================================*
 *                                 dr_reset
 *=======================================================================*/
PUBLIC
int	dr_reset()
{
int returnval;
int  recchar,j,a,d,i,b1,b2,b3,b4,b5,b6;
int c0,c1,c2,c3,c4,c5,c6;
long ll;
int parity;
double freq;
int adds[10];

recchar=33;
adds[0]=0;
adds[1]=1;
adds[2]=2;
adds[3]=3;
adds[4]=4;
adds[5]=5;
adds[6]=6;

freq=14.7456*1.1;
if (Time_set)
	{
	printf("CLOCK FREQUENCY CORRECTION\n\n");
	reset_digirec();
	if(!dr_sync_after_reset()) er_abort(DR_NO_DIGIREC);
	printf("\nType the actual frequency of the\n  oscillator in mHz (e.g. 14.7456):\n");
	scanf("%lf",&freq);
	compute_time_nums(freq,&b1,&b2,&b3,&b4,&b5,&b6);
	if (Debug_enabled)
	printf("b0=%02x, b1=%02x, b2=%02x, b3=%02x, b4=%02x, b5=%02x, b6=%02x\n",
		recchar,b1,b2,b3,b4,b5,b6);
	c0=c1=c2=c3=c4=c5=c6=0;
	if ( !dr_nvwrite(adds[0],recchar) ) printf("nvw error b0\n");
	if ( !dr_nvwrite(adds[1],b1) ) printf("nvw error b1\n");
	if ( !dr_nvwrite(adds[2],b2) ) printf("nvw error b2\n");
	if ( !dr_nvwrite(adds[3],b3) ) printf("nvw error b3\n");
	if ( !dr_nvwrite(adds[4],b4) ) printf("nvw error b4\n");
	if ( !dr_nvwrite(adds[5],b5) ) printf("nvw error b5\n");
	if ( !dr_nvwrite(adds[6],b6) ) printf("nvw error b6\n");
	if (i=dr_nvread(adds[0],&c0) == 9) printf("nvr error %d c0\n",i);
	if (i=dr_nvread(adds[1],&c1) == 9) printf("nvr error %d c1\n",i);
	if (i=dr_nvread(adds[2],&c2) == 9) printf("nvr error %d c2\n",i);
	if (i=dr_nvread(adds[3],&c3) == 9) printf("nvr error %d c3\n",i);
	if (i=dr_nvread(adds[4],&c4) == 9) printf("nvr error %d c4\n",i);
	if (i=dr_nvread(adds[5],&c5) == 9) printf("nvr error %d c5\n",i);
	if (i=dr_nvread(adds[6],&c6) == 9) printf("nvr error %d c6\n",i);
	if (Debug_enabled)
		printf("c0=%02x, c1=%02x, c2=%02x, c3=%02x, c4=%02x, c5=%02x, c6=%02x\n",
			c0,c1,c2,c3,c4,c5,c6);
	printf("Press any key to stop\n");
	while(!kbhit());
	getch();
	exit(0);
	}

reset_digirec();
if(!dr_sync_after_reset()) er_abort(DR_NO_DIGIREC);

if (Global_parity == 0)
	parity=NO_PARITY;
else if (Global_parity == -1)
	parity=ODD_PARITY;
else
	parity=EVEN_PARITY;

for ( i=0 ; i<16 ; i++)
	{
	dr_set_sync_char(i,33);
	dr_set_parity(i,parity);
	}

/* retrieve 6 frequency bytes from nvram  and recognition character */
i=dr_nvread(adds[0],&c0);
c1 = dr_nvread(adds[1],&b1);
c2 = dr_nvread(adds[2],&b2);
c3 = dr_nvread(adds[3],&b3);
c4 = dr_nvread(adds[4],&b4);
c5 = dr_nvread(adds[5],&b5);
c6 = dr_nvread(adds[6],&b6);

/* if we do not trust nvram data, compute standard frequency bytes */
if ((!i) || (c0!=recchar) || !c1 || !c2 || !c3 || !c4 || !c5 || !c6)
	{
	freq=14.7456;
	compute_time_nums(freq,&b1,&b2,&b3,&b4,&b5,&b6);
	}

/* adjust digirec clock */
dr_set_clock(b1,b2,b3,b4,b5,b6);

return(TRUE);
}

/*=======================================================================*
 *                                 dr_send_command
 *=======================================================================*/
PRIVATE
int dr_send_command(comm,parm)
int	comm;
int	parm;
{
unsigned int status;

lastcomm=comm;
output_word(digirec,((comm<<8)+(parm&0xff)));
status=input_word(digirec);
while((((status>>8)&0xff) != comm) && !kbhit())
	{
	output_word(digirec,((comm<<8)+(parm&0xff)));
	status=input_word(digirec);
	}
return(status&0xff);
}

/*=======================================================================*
 *                               dr_set_baud_rate
 *=======================================================================*/
dr_set_baud_rate(comm,parm)
int comm;
int parm;
{

if ((comm> -1)&&(comm<16))
	{
	switch(parm)
		{
		case B300:
		case B600:
		case B1200:
		case B2400:
		case B4800:
		case B9600:
			break;
		default:
			return(FALSE);
		}
	dr_issue_command(SHIFT_IN_BYTE,parm);
	dr_issue_command(SET_BAUD_RATE,comm);
	return(TRUE);
	}
else
	return(FALSE);
}

/*=======================================================================*
 *                               dr_set_clock
 *=======================================================================*/
dr_set_clock(b1,b2,b3,b4,b5,b6)
int b1,b2,b3,b4,b5,b6;
{
int lowcount,highcount,incrmt;

	dr_issue_command(SHIFT_IN_BYTE,b6);
	dr_issue_command(SHIFT_IN_BYTE,b5);
	dr_issue_command(SHIFT_IN_BYTE,b4);
	dr_issue_command(SHIFT_IN_BYTE,b3);
	dr_issue_command(SHIFT_IN_BYTE,b2);
	dr_issue_command(SHIFT_IN_BYTE,b1);
	dr_issue_command(SET_CLOCK,0);
	return(TRUE);
}

/*=======================================================================*
 *                                 dr_set_dma_chan
 *=======================================================================*/
PUBLIC
int dr_set_dma_chan(chan)
int chan;
{

dmachan=chan;

switch(chan)
	{
	case 5:
		dmaflag=ENABLE_DMA_CHANNEL_5;
		break;
	case 6:
		dmaflag=ENABLE_DMA_CHANNEL_6;
		break;
	case 7:
		dmaflag=ENABLE_DMA_CHANNEL_7;
		break;
	default:
		dmaflag=DISABLE_DMA;
		return(FALSE);
		break;
	}

return(TRUE);
}

/*=======================================================================*
 *                                 dr_set_int_chan
 *=======================================================================*/
PUBLIC
int dr_set_int_chan(chan)
int chan;
{

intchan=chan;
/* check interrupt port */
switch(intchan)
	{
	case 10:
		intflag=INT_10;
		break;
	case 15:
		intflag=INT_15;
		break;
	default:
		intflag=NO_INT;
		break;
	}

}

/*=======================================================================*
 *                                 dr_set_io_add
 *=======================================================================*/
PUBLIC
int dr_set_io_add(add)
int add;
{
digirec=add;
}

/*=======================================================================*
 *                               dr_set_parity
 *=======================================================================*/
PUBLIC
int dr_set_parity(comm,parm)
int comm;
int parm;
{

if ((comm> -1)&&(comm<16))
	{
	switch(parm)
		{
		case NO_PARITY:
		case EVEN_PARITY:
		case ODD_PARITY:
			break;
		default:
			return(FALSE);
		}
	dr_issue_command(SHIFT_IN_BYTE,parm);
	dr_issue_command(SET_PARITY,comm);
	return(TRUE);
	}
else
	return(FALSE);
}

/*=======================================================================*
 *                                 dr_set_position
 *=======================================================================*/
PUBLIC
void dr_set_position (chan,wrd,pos)
int chan;
int wrd;
int pos;
{
int	num,i,j;
if ((pos>63)||(pos<0)) pos=64;
dr_issue_command(SHIFT_IN_BYTE,pos);
dr_issue_command(SET_POSITION,((chan*4)+wrd));
}

/*=======================================================================*
 *                               dr_set_sync_char
 *=======================================================================*/
dr_set_sync_char(comm,parm)
int comm;
int parm;
{

if ((comm> -1)&&(comm<16))
	{
	dr_issue_command(SHIFT_IN_BYTE,parm);
	dr_issue_command(SET_SYNC,comm);
	return(TRUE);
	}
else
	return(FALSE);
}

/*=======================================================================*
 *                               dr_start_acq                           *
 *=======================================================================*/
PUBLIC
dr_start_acq()
{
int errflag,i,count,instrcnt;
long temp;

if (dmaflag == DISABLE_DMA) er_abort(DR_DMA_WRONG_CHANNEL);
if (intflag == NO_INT) er_abort(DR_INT_WRONG_CHANNEL);

/* turn off digirec dma */
instrcnt=dr_issue_command(SET_DMA,DISABLE_DMA);

/* enable digirec to cause interrupt */
dr_issue_command(GENERAL,DISABLE_INT); /*THIS CLEARS INTERRUPT FLAG */
dr_issue_command(SET_INT,intflag);
dr_issue_command(GENERAL,ENABLE_INT);


/* enable interrupt at the pc */
dr_irq_setup();

/* set up pc dma */
if( (dma_err = dr_start_dma()) != DMA_OKAY)
	{
	dr_stop_acq();
	er_abort(dma_err);
	}

/* turn on digirec dma */
dr_issue_command(SET_DMA,dmaflag);

}

/*=======================================================================*
 *                                 dr_start_dma
 *   memadd is a 32 bit address (only 24 bits usable)
 *   count is up to 64K (0 == 64K)
 *=======================================================================*/
PUBLIC
int dr_start_dma()
{
long memadd;
unsigned int count;
int adjustedcount;

/* allow only channels 5,6 and 7 for now */
if (dmaflag==DISABLE_DMA) return(DR_DMA_WRONG_CHANNEL);

/* get new xm buffer */
if (ehead==NULL) return(DR_DMA_BLOCKED);
empty_to_collect();

memadd=collect->pointer;
count=collect->length;

/* temporarily disable channel */

outp(maskreg[dmachan],chdsb[dmachan]);


/* set up page register */
outp(pagereg[dmachan], memadd>>16);

/* reset byte flip/flop */
outp(byteff[dmachan],0);

/* load address */
outp(addr[dmachan],memadd>>1);	/* low byte first */
outp(addr[dmachan],memadd>>9);	/* high byte second */

/* load adjusted count */
adjustedcount=count-1;
outp(cntreg[dmachan],adjustedcount);
outp(cntreg[dmachan],adjustedcount>>8);

/* set direction */
outp(modereg[dmachan],inmode[dmachan]);

/* and enable dma */
outp(maskreg[dmachan],chenb[dmachan]);

return(DMA_OKAY);
}

/*=======================================================================*
 *                               dr_stop_acq
 *=======================================================================*/
dr_stop_acq()
{
/* turn off digirec dma */
dr_issue_command(SET_DMA,DISABLE_DMA);

/* turn of digirec interrupt */
dr_issue_command(SET_INT,NO_INT);

/* restore pc interrupt vectors */
dr_irq_restore();

/* disable dma */
dr_stop_dma();

return(TRUE);
}

/*=======================================================================*
 *                               dr_stop_dma
 *=======================================================================*/
PUBLIC
int dr_stop_dma()
{

/* allow only channels 5,6 and 7 for now */
if (dmaflag == DISABLE_DMA) return(DR_DMA_WRONG_CHANNEL);

outp(maskreg[dmachan],chdsb[dmachan]);
return(DMA_OKAY);
}

/*=======================================================================*
 *                                 dr_sync_after_reset
 *=======================================================================*/
PRIVATE
int dr_sync_after_reset()
{
int notimeout;
long timeout_count;

timeout_count=10000L;

/* send a nop until you see a nop return */
do	{
	output_word(digirec,(NOP<<8));
	timeout_count -= 1L;
	notimeout= (timeout_count != 0L);
	}
while ((notimeout) && ((input_word(digirec)>>8) != NOP));

lastcomm=NOP;
return (notimeout);
}

/*=======================================================================*
 *                           empty_to_collect
 *=======================================================================*/
PRIVATE
empty_to_collect()
{
if (ehead != NULL)
	{
	collect = ehead;
	ehead=ehead->next;
	collect->next=NULL;
	if (ehead == NULL)
		etail = NULL;
	return(TRUE);
	}
else
	return(FALSE);
}

/*=======================================================================*
 *                                 fpta (far pointer to absolute)
 *=======================================================================*/
PUBLIC
long fpta (point)
int far *point;
{

return( ( ( (long)point & 0xffff0000L) >> 12 ) + ( (long)point & 0xffff) );
}

/*=======================================================================*
 *                           full_to_process
 *=======================================================================*/
PRIVATE
full_to_process()
{
if (fhead != NULL)
	{
	process = fhead;
	fhead=fhead->next;
	process->next = NULL;
	if (fhead == NULL)
		ftail = NULL;
	return(TRUE);
	}
else
	return(FALSE);
}

/*=======================================================================*
 *                        nv_try_write
 *=======================================================================*/
nv_try_write(add,da)
int add;
int da;
{
dr_issue_command(SHIFT_IN_BYTE,(add>>8));
dr_issue_command(SHIFT_IN_BYTE,add);

return(dr_issue_command(WRITE_NVR,da));

}

/*=======================================================================*
 *                           process_to_empty
 *=======================================================================*/
PRIVATE
process_to_empty()
{
if (process != NULL)
	{
	if (etail==NULL)
		{
		ehead=process;
		}
	else
		{
		etail->next=process;
		}
	etail=process;
	etail->next=NULL;
	process=NULL;
	return(TRUE);
	}
else
	return(FALSE);
}

/*=======================================================================*
 *                                 reset_digirec
 *=======================================================================*/
PRIVATE
int	reset_digirec()
{
input_word(digirec+4);
}

/*=======================================================================*
 *                                 _xmalloc
 *=======================================================================*/
PRIVATE
unsigned long _xmalloc(byte_count)
unsigned long byte_count;
{
long temp,page_bottom;

/* if count is greater than 128K, exit with 0 start address */
if (byte_count > 0x00020000L) return (0L);

/* if count is greater than remaining words in current 128K page,
	go to next lower page */
page_bottom=xm_hi_add & 0xfffe0000L;
if ((xm_hi_add - page_bottom)<byte_count) 
	{
	page_bottom -= byte_count;
	/* if not enough room left, send back 0 */
	if (page_bottom < xm_lo_add)
		return(0L);
	else /* cross page boundary and allocate buffer */
		{
		xm_hi_add = page_bottom;
		xm_pages_used += 1;
		return(xm_hi_add);
		}
	}
else
	{
	xm_hi_add -= byte_count;
	if (xm_hi_add < xm_lo_add)
		{
		xm_hi_add += byte_count;
		return(0L);
		}
	else
		return(xm_hi_add);
	}
}

